@@ -1,21 +1,27 @@
 # add software-center custom metadata to the index
 
 import apt
-import re
 import os
 import sys
 import xapian
 
 sys.path.insert(0, "/usr/share/software-center")
-from softwarecenter.db.update import *
+from softwarecenter.enums import (
+    CustomKeys,
+    XapianValues,
+    )
+from softwarecenter.db.update import index_name
+from softwarecenter.distro import get_distro
+
 
 class SoftwareCenterMetadataPlugin:
+
     def info(self):
         """
         Return general information about the plugin.
 
         The information returned is a dict with various keywords:
-         
+
          timestamp (required)
            the last modified timestamp of this data source.  This will be used
            to see if we need to update the database or not.  A timestamp of 0
@@ -30,8 +36,8 @@ class SoftwareCenterMetadataPlugin:
         """
         file = apt.apt_pkg.config.find_file("Dir::Cache::pkgcache")
         if not os.path.exists(file):
-            return dict(timestamp = 0)
-        return dict(timestamp = os.path.getmtime(file))
+            return dict(timestamp=0)
+        return dict(timestamp=os.path.getmtime(file))
 
     def init(self, info, progress):
         """
@@ -56,24 +62,23 @@ class SoftwareCenterMetadataPlugin:
           fullDoc: the full description as a chapter in ReST format
         """
         return dict(
-            name = "SoftwareCenterMetadata",
-            shortDesc = "SoftwareCenter meta information",
-            fullDoc = """
-            Software-center metadata 
+            name="SoftwareCenterMetadata",
+            shortDesc="SoftwareCenter meta information",
+            fullDoc="""
+            Software-center metadata
             It uses the prefixes:
               AA for the Application name
               AP for the Package name
               AC for the categories
               AT to "application" for applications
-            It sets the following xapian values from the software-center 
+            It sets the following xapian values from the software-center
             enums:
-              XAPIAN_VALUE_ICON
-              XAPIAN_VALUE_ICON_NEEDS_DOWNLOAD
-              XAPIAN_VALUE_SCREENSHOT_URL
-              XAPIAN_VALUE_THUMBNAIL_URL
-            """
-        )
-
+              XapianValues.ICON
+              XapianValues.ICON_NEEDS_DOWNLOAD
+              XapianValues.ICON_URL
+              XapianValues.SCREENSHOT_URLS
+              XapianValues.THUMBNAIL_URL
+            """)
 
     def index(self, document, pkg):
         """
@@ -85,33 +90,48 @@ class SoftwareCenterMetadataPlugin:
         ver = pkg.candidate
         # if there is no version or the AppName custom key is not
         # found we can skip the pkg
-        if ver is None or not CUSTOM_KEY_APPNAME in ver.record:
+        if ver is None or not CustomKeys.APPNAME in ver.record:
             return
-        # we want to index the following custom fields: 
-        #   XB-AppName, XB-Icon, XB-Screenshot-Url, XB-Thumbnail-Url, XB-Category
-        if CUSTOM_KEY_APPNAME in ver.record:
-            name = ver.record[CUSTOM_KEY_APPNAME]
+        # we want to index the following custom fields:
+        #   XB-AppName,
+        #   XB-Icon,
+        #   XB-Screenshot-Url,
+        #   XB-Thumbnail-Url,
+        #   XB-Category
+        if CustomKeys.APPNAME in ver.record:
+            name = ver.record[CustomKeys.APPNAME]
             self.indexer.set_document(document)
             index_name(document, name, self.indexer)
             # we pretend to be an application
-            document.add_term("AT"+"application")
+            document.add_term("AT" + "application")
             # and we inject a custom component value to indicate "independent"
-            document.add_value(XAPIAN_VALUE_ARCHIVE_SECTION, "independent")
-        if CUSTOM_KEY_ICON in ver.record:
-            icon = ver.record[CUSTOM_KEY_ICON]
-            document.add_value(XAPIAN_VALUE_ICON, icon)
-            document.add_value(XAPIAN_VALUE_ICON_NEEDS_DOWNLOAD, "1")
-        if CUSTOM_KEY_SCREENSHOT_URL in ver.record:
-            screenshot_url = ver.record[CUSTOM_KEY_SCREENSHOT_URL]
-            document.add_value(XAPIAN_VALUE_SCREENSHOT_URL, screenshot_url)
-        if CUSTOM_KEY_THUMBNAIL_URL in ver.record:
-            url = ver.record[CUSTOM_KEY_THUMBNAIL_URL]
-            document.add_value(XAPIAN_VALUE_THUMBNAIL_URL, url)
-        if CUSTOM_KEY_CATEGORY in ver.record:
-            categories_str = ver.record[CUSTOM_KEY_CATEGORY]
+            document.add_value(XapianValues.ARCHIVE_SECTION, "independent")
+        if CustomKeys.ICON in ver.record:
+            icon = ver.record[CustomKeys.ICON]
+            document.add_value(XapianValues.ICON, icon)
+            # calculate the url and add it (but only if there actually is
+            # a url)
+            try:
+                distro = get_distro()
+                if distro:
+                    base_uri = ver.uri
+                    # new python-apt returns None instead of StopIteration
+                    if base_uri:
+                        url = distro.get_downloadable_icon_url(base_uri, icon)
+                        document.add_value(XapianValues.ICON_URL, url)
+            except StopIteration:
+                pass
+        if CustomKeys.SCREENSHOT_URLS in ver.record:
+            screenshot_url = ver.record[CustomKeys.SCREENSHOT_URLS]
+            document.add_value(XapianValues.SCREENSHOT_URLS, screenshot_url)
+        if CustomKeys.THUMBNAIL_URL in ver.record:
+            url = ver.record[CustomKeys.THUMBNAIL_URL]
+            document.add_value(XapianValues.THUMBNAIL_URL, url)
+        if CustomKeys.CATEGORY in ver.record:
+            categories_str = ver.record[CustomKeys.CATEGORY]
             for cat in categories_str.split(";"):
                 if cat:
-                    document.add_term("AC"+cat.lower())
+                    document.add_term("AC" + cat.lower())
 
     def indexDeb822(self, document, pkg):
         """
