@@ -21,6 +21,8 @@ import os
 import logging
 import xapian
 
+import softwarecenter.paths
+
 from gi.repository import GObject
 
 from aptsources.sourceslist import SourceEntry, SourcesList
@@ -37,17 +39,18 @@ from softwarecenter.enums import (ViewPages,
 
 LOG = logging.getLogger(__name__)
 
+
 class AptChannelsManager(ChannelsManager):
 
     def __init__(self, db):
         self.db = db
         self.distro = get_distro()
         self.backend = get_install_backend()
-        self.backend.connect("channels-changed", 
+        self.backend.connect("channels-changed",
                              self._remove_no_longer_needed_extra_channels)
         # kick off a background check for changes that may have been made
         # in the channels list
-        GObject.timeout_add_seconds(10, self._check_for_channel_updates_timer)
+        GObject.timeout_add_seconds(60, self._check_for_channel_updates_timer)
         # extra channels from e.g. external sources
         self.extra_channels = []
         self._logger = LOG
@@ -58,7 +61,7 @@ class AptChannelsManager(ChannelsManager):
         """
         return a list of SoftwareChannel objects in display order
         according to:
-            Distribution, Partners, PPAs alphabetically, 
+            Distribution, Partners, PPAs alphabetically,
             Other channels alphabetically, Unknown channel last
         """
         return self._get_channels()
@@ -68,7 +71,7 @@ class AptChannelsManager(ChannelsManager):
         """
         return a list of SoftwareChannel objects displaying installed
         packages only in display order according to:
-            Distribution, Partners, PPAs alphabetically, 
+            Distribution, Partners, PPAs alphabetically,
             Other channels alphabetically, Unknown channel last
         """
         return self._get_channels(installed_only=True)
@@ -87,7 +90,7 @@ class AptChannelsManager(ChannelsManager):
         return the new channel object
         """
         # print name, icon, query
-        channel = SoftwareChannel(name, None, None, 
+        channel = SoftwareChannel(name, None, None,
                                   channel_icon=icon,
                                   channel_query=query)
         self.extra_channels.append(channel)
@@ -111,7 +114,7 @@ class AptChannelsManager(ChannelsManager):
         """
         this feeds in a private sources.list entry that is
         available to the user (like a private PPA) that may or
-        may not be active 
+        may not be active
         """
         # FIXME: strip out password and use apt/auth.conf
         potential_new_entry = SourceEntry(source_entry)
@@ -148,47 +151,30 @@ class AptChannelsManager(ChannelsManager):
 
     def _check_for_channel_updates_timer(self):
         """
-        run a background timer to see if the a-x-i data we have is 
+        run a background timer to see if the a-x-i data we have is
         still fresh or if the cache has changed since
         """
-        if not self.db._aptcache.ready:
-            return True
-        # see if we need a a-x-i update
-        if self._check_for_channel_updates():
-            # this will trigger a "channels-changed" signal from
-            # the backend object once a-x-i is finished
-            self._logger.debug("running update_xapian_index")
-            self.backend.update_xapian_index()
-        return False
+        # this is expensive and does not need UI to we shove it out
+        channel_update = os.path.join(
+            softwarecenter.paths.datadir, "update-software-center-channels")
+        (pid, stdin, stdout, stderr) = GObject.spawn_async(
+            [channel_update],
+            flags=GObject.SPAWN_DO_NOT_REAP_CHILD)
+        GObject.child_watch_add(
+            pid, self._on_check_for_channel_updates_finished)
+
+    def _on_check_for_channel_updates_finished(self, pid, condition):
+        # exit status of 1 means stuff changed
+        if os.WEXITSTATUS(condition) == 1:
+            self.db.reopen()
 
-    def _check_for_channel_updates(self):
-        """ 
-        check current set of channel origins in a-x-i and
-        compare it to the apt cache to see if 
-        anything has changed, 
-        
-        returns True is a update is needed
-        """
-        # the operation get_origins can take some time (~60s?)
-        cache_origins = set(self.db._aptcache.get_all_origins())
-        db_origins = set(self.db.get_origins_from_db())
-        # origins
-        self._logger.debug("cache_origins: %s" % cache_origins)
-        self._logger.debug("db_origins: %s" % db_origins)
-        # the db_origins will contain origins from the s-c-agent, so
-        # we don't need to rebuild if the db has more origins then
-        # the cache, but we need to rebuild if the cache has origins
-        # that a-x-i does not have
-        if not cache_origins.issubset(db_origins):
-            return True
-        return False
-    
     def _get_channels(self, installed_only=False):
         """
-        (internal) implements 'channels()' and 'channels_installed_only()' properties
+        (internal) implements 'channels()' and 'channels_installed_only()'
+        properties
         """
         distro_channel_name = self.distro.get_distro_channel_name()
-        
+
         # gather the set of software channels and order them
         other_channel_list = []
         cached_origins = []
@@ -197,12 +183,13 @@ class AptChannelsManager(ChannelsManager):
                 continue
             channel_name = channel_iter.term[3:]
             channel_origin = ""
-            
+
             # get origin information for this channel
             m = self.db.xapiandb.postlist_begin(channel_iter.term)
             doc = self.db.xapiandb.get_document(m.get_docid())
             for term_iter in doc.termlist():
-                if term_iter.term.startswith("XOO") and len(term_iter.term) > 3: 
+                if (term_iter.term.startswith("XOO") and
+                      len(term_iter.term) > 3):
                     channel_origin = term_iter.term[3:]
                     break
             self._logger.debug("channel_name: %s" % channel_name)
@@ -210,7 +197,7 @@ class AptChannelsManager(ChannelsManager):
             if channel_origin not in cached_origins:
                 other_channel_list.append((channel_name, channel_origin))
                 cached_origins.append(channel_origin)
-        
+
         dist_channel = None
         partner_channel = None
         for_purchase_channel = None
@@ -223,9 +210,9 @@ class AptChannelsManager(ChannelsManager):
         for (channel_name, channel_origin) in other_channel_list:
             if not channel_name:
                 unknown_channel.append(SoftwareChannel(channel_name,
-                                                       channel_origin,
-                                                       None,
-                                                       installed_only=installed_only))
+                    channel_origin,
+                    None,
+                    installed_only=installed_only))
             elif channel_name == distro_channel_name:
                 dist_channel = (SoftwareChannel(distro_channel_name,
                                                 channel_origin,
@@ -233,59 +220,62 @@ class AptChannelsManager(ChannelsManager):
                                                 installed_only=installed_only))
             elif channel_name == "Partner archive":
                 partner_channel = SoftwareChannel(channel_name,
-                                                  channel_origin,
-                                                  "partner", 
-                                                  installed_only=installed_only)
+                    channel_origin,
+                    "partner",
+                    installed_only=installed_only)
             elif channel_name == "notdownloadable":
                 if installed_only:
                     local_channel = SoftwareChannel(channel_name,
-                                                    None,
-                                                    None,
-                                                    installed_only=installed_only)
+                        None,
+                        None,
+                        installed_only=installed_only)
             elif (channel_origin and
-                  channel_origin.startswith("LP-PPA-commercial-ppa-uploaders")):
+                 channel_origin.startswith("LP-PPA-commercial-ppa-uploaders")):
                 # do not display commercial private PPAs, they will all be
                 # displayed in the "for-purchase" node anyway
                 pass
             elif channel_origin and channel_origin.startswith("LP-PPA"):
                 if channel_origin == "LP-PPA-app-review-board":
                     new_apps_channel = SoftwareChannel(channel_name,
-                                                       channel_origin,
-                                                       None,
-                                                       installed_only=installed_only)
+                        channel_origin,
+                        None,
+                        installed_only=installed_only)
                 else:
                     ppa_channels.append(SoftwareChannel(channel_name,
-                                                        channel_origin,
-                                                        None,
-                                                        installed_only=installed_only))
+                        channel_origin,
+                        None,
+                        installed_only=installed_only))
             # TODO: detect generic repository source (e.g., Google, Inc.)
             else:
                 other_channels.append(SoftwareChannel(channel_name,
-                                                      channel_origin,
-                                                      None,
-                                                      installed_only=installed_only))
+                    channel_origin,
+                    None,
+                    installed_only=installed_only))
 
-        # always display the partner channel, even if its source is not enabled                                                       
-        if not partner_channel:
+        # always display the partner channel, even if its source is not enabled
+        if not partner_channel and distro_channel_name == "Ubuntu":
             partner_channel = SoftwareChannel("Partner archive",
                                               "Canonical",
-                                              "partner", 
+                                              "partner",
                                               installed_only=installed_only)
-        
-        # create a "magic" channel to display items available for purchase                                              
-        for_purchase_query = xapian.Query("AH" + AVAILABLE_FOR_PURCHASE_MAGIC_CHANNEL_NAME)
-        for_purchase_channel = SoftwareChannel("For Purchase", None, None, 
-                                               channel_icon=None,   # FIXME:  need an icon
-                                               channel_query=for_purchase_query,
-                                               installed_only=installed_only)
-        
+
+        # create a "magic" channel to display items available for purchase
+        for_purchase_query = xapian.Query("AH" +
+            AVAILABLE_FOR_PURCHASE_MAGIC_CHANNEL_NAME)
+        for_purchase_channel = SoftwareChannel("For Purchase",
+            "software-center-agent", None,
+            channel_icon=None,  # FIXME:  need an icon
+            channel_query=for_purchase_query,
+            installed_only=installed_only)
+
         # set them in order
         channels = []
         if dist_channel is not None:
             channels.append(dist_channel)
         if partner_channel is not None:
             channels.append(partner_channel)
-        channels.append(for_purchase_channel)
+        if get_distro().PURCHASE_APP_URL:
+            channels.append(for_purchase_channel)
         if new_apps_channel is not None:
             channels.append(new_apps_channel)
         channels.extend(ppa_channels)
@@ -301,4 +291,3 @@ class AptChannelsManager(ChannelsManager):
             else:
                 channel._channel_view_id = ViewPages.AVAILABLE
         return channels
-
