@@ -16,30 +16,36 @@
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from gi.repository import GObject, Gio
+
+import json
 import locale
 import logging
 import os
 import re
 
+import softwarecenter.distro
+
 from gettext import gettext as _
 from softwarecenter.backend.channel import is_channel_available
-from softwarecenter.distro import get_distro
 from softwarecenter.enums import PkgStates, XapianValues, Icons
 
 from softwarecenter.paths import (APP_INSTALL_CHANNELS_PATH,
                                   SOFTWARE_CENTER_ICON_CACHE_DIR,
                                   )
-from softwarecenter.utils import utf8, split_icon_ext
+from softwarecenter.utils import utf8, split_icon_ext, capitalize_first_word
+from softwarecenter.region import get_region_cached, REGIONTAG
 
 LOG = logging.getLogger(__name__)
 
+
 # this is a very lean class as its used in the main listview
 # and there are a lot of application objects in memory
 class Application(object):
-    """ The central software item abstraction. it contains a 
+    """ The central software item abstraction. it contains a
         pkgname that is always available and a optional appname
         for packages with multiple applications
-        
+
         There is also a __cmp__ method and a name property
     """
     def __init__(self, appname="", pkgname="", request="", popcon=0):
@@ -54,6 +60,10 @@ class Application(object):
         # the request can take additional "request" data like apturl
         # strings or the path of a local deb package
         self.request = request
+        # a archive_suite can be used to force a specific version that
+        # would not be installed automatically (like ubuntu-backports)
+        self.archive_suite = ""
+        # popcon
         self._popcon = popcon
         # a "?" in the name means its a apturl request
         if "?" in pkgname:
@@ -66,9 +76,11 @@ class Application(object):
         if self.appname:
             return self.appname
         return self.pkgname
+
     @property
     def popcon(self):
         return self._popcon
+
     # get a AppDetails object for this Applications
     def get_details(self, db):
         """ return a new AppDetails object for this application """
@@ -76,13 +88,14 @@ class Application(object):
 
     def get_untranslated_app(self, db):
         """ return a Application object with the untranslated application
-            name 
+            name
         """
         try:
             doc = db.get_xapian_document(self.appname, self.pkgname)
         except IndexError:
             return self
-        untranslated_application = doc.get_value(XapianValues.APPNAME_UNTRANSLATED)
+        untranslated_application = doc.get_value(
+            XapianValues.APPNAME_UNTRANSLATED)
         uapp = Application(untranslated_application, self.pkgname)
         return uapp
 
@@ -95,30 +108,39 @@ class Application(object):
         if doc:
             appname = db.get_appname(doc)
             if appname:
+                if db.is_appname_duplicated(appname):
+                    appname = "%s (%s)" % (appname, db.get_pkgname(doc))
                 return appname
             else:
-                return db.get_summary(doc)
+                return capitalize_first_word(db.get_summary(doc))
+
     @staticmethod
     def get_display_summary(db, doc):
         """ Return the application summary as it should be displayed in the UI
-            If the appname is defined, return the application summary, else return
-            the application's pkgname (per the spec)
+            If the appname is defined, return the application summary, else
+            return the application's pkgname (per the spec)
         """
         if doc:
             if db.get_appname(doc):
-                return db.get_summary(doc)
+                return capitalize_first_word(db.get_summary(doc))
             else:
                 return db.get_pkgname(doc)
-        
+
     # special methods
     def __hash__(self):
-        return ("%s:%s" % (self.appname, self.pkgname)).__hash__()
+        return utf8("%s:%s" % (
+                utf8(self.appname), utf8(self.pkgname))).__hash__()
+
     def __cmp__(self, other):
         return self.apps_cmp(self, other)
+
     def __str__(self):
         return utf8("%s,%s") % (utf8(self.appname), utf8(self.pkgname))
+
     def __repr__(self):
-        return "[Application: appname=%s pkgname=%s]" % (self.appname, self.pkgname)
+        return "[Application: appname=%s pkgname=%s]" % (self.appname,
+            self.pkgname)
+
     @staticmethod
     def apps_cmp(x, y):
         """ sort method for the applications """
@@ -133,22 +155,30 @@ class Application(object):
         else:
             return cmp(x.pkgname, y.pkgname)
 
+
 # the details
-class AppDetails(object):
+class AppDetails(GObject.GObject):
     """ The details for a Application. This contains all the information
         we have available like website etc
     """
 
+    __gsignals__ = {"screenshots-available": (GObject.SIGNAL_RUN_FIRST,
+                                              GObject.TYPE_NONE,
+                                              (GObject.TYPE_PYOBJECT,),
+                                             ),
+                    }
+
     def __init__(self, db, doc=None, application=None):
         """ Create a new AppDetails object. It can be created from
             a xapian.Document or from a db.application.Application object
         """
+        GObject.GObject.__init__(self)
         if not doc and not application:
             raise ValueError("Need either document or application")
         self._db = db
         self._db.connect("reopen", self._on_db_reopen)
         self._cache = self._db._aptcache
-        self._distro = get_distro()
+        self._distro = softwarecenter.distro.get_distro()
         self._history = None
         # import here (intead of global) to avoid dbus dependency
         # in update-software-center (that imports application, but
@@ -158,12 +188,13 @@ class AppDetails(object):
         # FIXME: why two error states ?
         self._error = None
         self._error_not_found = None
+        self._screenshot_list = []
 
         # load application
         self._app = application
         if doc:
-            self._app = Application(self._db.get_appname(doc), 
-                                    self._db.get_pkgname(doc), 
+            self._app = Application(self._db.get_appname(doc),
+                                    self._db.get_pkgname(doc),
                                     "")
 
         # sustitute for apturl
@@ -173,7 +204,7 @@ class AppDetails(object):
 
         # load pkg cache
         self._pkg = None
-        if (self._app.pkgname in self._cache and 
+        if (self._app.pkgname in self._cache and
             self._cache[self._app.pkgname].candidate):
             self._pkg = self._cache[self._app.pkgname]
 
@@ -187,16 +218,19 @@ class AppDetails(object):
                 # if there is no document and no apturl request,
                 # set error state
                 debfile_matches = re.findall(r'/', self._app.request)
-                channel_matches = re.findall(r'channel=[a-z,-]*', 
+                channel_matches = re.findall(r'channel=[a-z,-]*',
                                              self._app.request)
-                section_matches = re.findall(r'section=[a-z]*', 
+                section_matches = re.findall(r'section=[a-z]*',
                                              self._app.request)
-                if (not self._pkg and 
-                    not debfile_matches and 
-                    not channel_matches and 
+                if (not self._pkg and
+                    not debfile_matches and
+                    not channel_matches and
                     not section_matches):
                     self._error = _("Not found")
-                    self._error_not_found = utf8(_(u"There isn\u2019t a software package called \u201c%s\u201D in your current software sources.")) % utf8(self.pkgname)
+                    self._error_not_found = utf8(
+                        _(u"There isn\u2019t a "
+                          u"software package called \u201c%s\u201D in your "
+                          u"current software sources.")) % utf8(self.pkgname)
 
     def same_app(self, other):
         return self.pkgname == other.pkgname
@@ -211,6 +245,18 @@ class AppDetails(object):
                 LOG.warn("document no longer valid after db reopen")
                 self._doc = None
 
+    def _get_version_for_archive_suite(self, pkg, archive_suite):
+        """ helper for the multiple versions support """
+        if not archive_suite:
+            return pkg.candidate
+        else:
+            for ver in pkg.versions:
+                archive_suites = [origin.archive for origin in ver.origins]
+                if archive_suite in archive_suites:
+                    return ver
+        raise ValueError("pkg '%s' has not archive_suite '%s'" % (
+                pkg, archive_suite))
+
     @property
     def channelname(self):
         if self._doc:
@@ -220,7 +266,8 @@ class AppDetails(object):
                 return channel
         else:
             # check if we have an apturl request to enable a channel
-            channel_matches = re.findall(r'channel=([0-9a-z,-]*)', self._app.request)
+            channel_matches = re.findall(r'channel=([0-9a-z,-]*)',
+                self._app.request)
             if channel_matches:
                 channel = channel_matches[0]
                 channelfile = APP_INSTALL_CHANNELS_PATH + channel + ".list"
@@ -237,22 +284,23 @@ class AppDetails(object):
     def eulafile(self):
         channel = self.channelname
         if channel:
-            eulafile =  APP_INSTALL_CHANNELS_PATH + channel + ".eula"
+            eulafile = APP_INSTALL_CHANNELS_PATH + channel + ".eula"
             if os.path.exists(eulafile):
                 return eulafile
 
     @property
     def component(self):
-        """ 
+        """
         get the component (main, universe, ..)
-        
-        this uses the data from apt, if there is none it uses the 
+
+        this uses the data from apt, if there is none it uses the
         data from the app-install-data files
         """
         # try apt first
         if self._pkg:
             for origin in self._pkg.candidate.origins:
-                if (origin.origin == "Ubuntu" and origin.trusted and origin.component):
+                if (origin.origin == self._distro.get_distro_channel_name() and
+                    origin.trusted and origin.component):
                     return origin.component
         # then xapian
         elif self._doc:
@@ -260,11 +308,14 @@ class AppDetails(object):
             return comp
         # then apturl requests
         else:
-            section_matches = re.findall(r'section=([a-z]+)', self._app.request)
+            section_matches = re.findall(r'section=([a-z]+)',
+                self._app.request)
             if section_matches:
                 valid_section_matches = []
                 for section_match in section_matches:
-                    if self._unavailable_component(component_to_check=section_match) and valid_section_matches.count(section_match) == 0:
+                    if (self._unavailable_component(
+                        component_to_check=section_match) and
+                        valid_section_matches.count(section_match) == 0):
                         valid_section_matches.append(section_match)
                 if valid_section_matches:
                     return ('&').join(valid_section_matches)
@@ -277,7 +328,9 @@ class AppDetails(object):
     @property
     def description(self):
         if self._pkg:
-            return self._pkg.candidate.description
+            ver = self._get_version_for_archive_suite(
+                self._pkg, self._app.archive_suite)
+            return ver.description
         elif self._doc:
             if self._doc.get_value(XapianValues.SC_DESCRIPTION):
                 return self._doc.get_value(XapianValues.SC_DESCRIPTION)
@@ -295,8 +348,11 @@ class AppDetails(object):
             return self._error
         # this may have changed since we inited the appdetails
         elif self.pkg_state == PkgStates.NOT_FOUND:
-            self._error =  _("Not found")
-            self._error_not_found = utf8(_(u"There isn\u2019t a software package called \u201c%s\u201D in your current software sources.")) % utf8(self.pkgname)
+            self._error = _("Not found")
+            self._error_not_found = utf8(
+                _(u"There isn\u2019t a software "
+                  u"package called \u201c%s\u201D in your current software "
+                  u"sources.")) % utf8(self.pkgname)
             return self._error_not_found
 
     @property
@@ -307,12 +363,12 @@ class AppDetails(object):
             return split_icon_ext(self._db.get_iconname(self._doc))
         if not self.summary:
             return Icons.MISSING_PKG
-            
+
     @property
     def icon_file_name(self):
         if self._doc:
             return self._db.get_iconname(self._doc)
-    
+
     @property
     def icon_url(self):
         if self._doc:
@@ -321,7 +377,8 @@ class AppDetails(object):
     @property
     def cached_icon_file_path(self):
         if self._doc:
-            return os.path.join(SOFTWARE_CENTER_ICON_CACHE_DIR, self._db.get_iconname(self._doc))
+            return os.path.join(SOFTWARE_CENTER_ICON_CACHE_DIR,
+                self._db.get_iconname(self._doc))
 
     @property
     def installation_date(self):
@@ -344,6 +401,8 @@ class AppDetails(object):
             # translations, but fortunately for the most common ones
             # like "Properitary" we have translations in s-c
             return _(xapian_license)
+        elif self._pkg and self._pkg.license:
+            return self._pkg.license
         else:
             return self._distro.get_license_text(self.component)
 
@@ -355,7 +414,7 @@ class AppDetails(object):
     @property
     def maintenance_status(self):
         return self._distro.get_maintenance_status(
-            self._cache, self.display_name, self.pkgname, self.component, 
+            self._cache, self.display_name, self.pkgname, self.component,
             self.channelname)
 
     @property
@@ -365,7 +424,7 @@ class AppDetails(object):
             the property display_name instead
         """
         return self._app.name
-    
+
     @property
     def display_name(self):
         """ Return the application name as it should be displayed in the UI
@@ -381,8 +440,8 @@ class AppDetails(object):
     @property
     def display_summary(self):
         """ Return the application summary as it should be displayed in the UI
-            If the appname is defined, return the application summary, else return
-            the application's pkgname (per the spec)
+            If the appname is defined, return the application summary, else
+            return the application's pkgname (per the spec)
         """
         if self._doc:
             return Application.get_display_summary(self._db, self._doc)
@@ -417,9 +476,17 @@ class AppDetails(object):
         # - available for download (via sources.list)
         # - locally installed
         # - intalled and available for download
+        # - installed but the user wants to switch versions between
+        #   not-automatic channels (like experimental/backports)
         if self._pkg:
-            # Don't handle upgrades yet
-            #if self._pkg.installed and self._pkg._isUpgradable:
+            if self._pkg.installed and self._app.archive_suite:
+                archive_suites = [origin.archive
+                                  for origin in self._pkg.installed.origins]
+                if not self._app.archive_suite in archive_suites:
+                    return PkgStates.FORCE_VERSION
+            # Don't handle upgrades yet, see bug LP #976525 we need more UI
+            # for this
+            #if self._pkg.installed and self._pkg.is_upgradable:
             #    return PkgStates.UPGRADABLE
             if self._pkg.is_installed:
                 return PkgStates.INSTALLED
@@ -436,22 +503,45 @@ class AppDetails(object):
                 if self._unavailable_channel():
                     return PkgStates.NEEDS_SOURCE
                 else:
-                    self._error =  _("Not found")
-                    self._error_not_found = utf8(_(u"There isn\u2019t a software package called \u201c%s\u201D in your current software sources.")) % utf8(self.pkgname)
+                    self._error = _("Not found")
+                    self._error_not_found = utf8(
+                        _(u"There isn\u2019t a "
+                          u"software package called \u201c%s\u201D in your "
+                          u"current software sources.")) % utf8(self.pkgname)
                     return PkgStates.NOT_FOUND
             else:
                 if self.price:
                     return PkgStates.NEEDS_PURCHASE
                 if (self.purchase_date and
                     self._doc.get_value(XapianValues.ARCHIVE_DEB_LINE)):
-                    return PkgStates.PURCHASED_BUT_REPO_MUST_BE_ENABLED
+                    supported_distros = self.supported_distros
+
+                    # Until bug 917109 is fixed on the server we won't have
+                    # any supported_distros for a for-purchase app, so we
+                    # follow the current behaviour in this case.
+                    if not supported_distros:
+                        return PkgStates.PURCHASED_BUT_REPO_MUST_BE_ENABLED
+
+                    current_distro = self._distro.get_codename()
+                    current_arch = self._distro.get_architecture()
+                    if current_distro in supported_distros and (
+                        current_arch in supported_distros[current_distro] or
+                        'any' in supported_distros[current_distro]):
+                        return PkgStates.PURCHASED_BUT_REPO_MUST_BE_ENABLED
+                    else:
+                        return PkgStates.PURCHASED_BUT_NOT_AVAILABLE_FOR_SERIES
+
                 if self.component:
                     components = self.component.split('&')
                     for component in components:
-                        if component and self._unavailable_component(component_to_check=component):
+                        if component and self._unavailable_component(
+                            component_to_check=component):
                             return PkgStates.NEEDS_SOURCE
-                self._error =  _("Not found")
-                self._error_not_found = utf8(_(u"There isn\u2019t a software package called \u201c%s\u201D in your current software sources.")) % utf8(self.pkgname)
+                self._error = _("Not found")
+                self._error_not_found = utf8(
+                    _(u"There isn\u2019t a software "
+                      u"package called \u201c%s\u201D in your current "
+                      u"software sources.")) % utf8(self.pkgname)
                 return PkgStates.NOT_FOUND
         return PkgStates.UNKNOWN
 
@@ -460,6 +550,16 @@ class AppDetails(object):
         if self._doc:
             return self._doc.get_value(XapianValues.PRICE)
 
+    @property
+    def supported_distros(self):
+        if self._doc:
+            supported_series = self._doc.get_value(
+                XapianValues.SC_SUPPORTED_DISTROS)
+            if not supported_series:
+                return {}
+
+            return json.loads(supported_series)
+
     @property
     def ppaname(self):
         if self._doc:
@@ -477,18 +577,131 @@ class AppDetails(object):
 
     @property
     def screenshot(self):
+        """ return screenshot url """
         # if there is a custom screenshot url provided, use that
         if self._doc:
-            if self._doc.get_value(XapianValues.SCREENSHOT_URL):
-                return self._doc.get_value(XapianValues.SCREENSHOT_URL)
+            # we do support multiple screenshots in the database but
+            # return only one here
+            screenshot_url = self._doc.get_value(XapianValues.SCREENSHOT_URLS)
+            if screenshot_url:
+                return screenshot_url.split(",")[0]
         # else use the default
-        return self._distro.SCREENSHOT_LARGE_URL % { 'pkgname' : self.pkgname, 
-                                                     'version' : self.version or 0 }
+        return self._distro.SCREENSHOT_LARGE_URL % {
+            'pkgname': self.pkgname,
+            'version': self.version or 0,
+        }
+
+    @property
+    def screenshots(self):
+        """ return list of screenshots, this requies that
+            "query_multiple_screenshos" was run before and emited the signal
+        """
+        if not self._screenshot_list:
+            return [{
+                'small_image_url': self.thumbnail,
+                'large_image_url': self.screenshot,
+                'version': self.version,
+            }]
+        return self._screenshot_list
+
+    @property
+    def tags(self):
+        """ return a set() of tags """
+        terms = set()
+        if self._doc:
+            for term_iter in self._doc.termlist():
+                if term_iter.term.startswith("XT"):
+                    terms.add(term_iter.term[2:])
+        return terms
+
+    def _get_multiple_screenshots_from_db(self):
+        screenshot_list = []
+        if self._doc:
+            screenshot_url = self._doc.get_value(XapianValues.SCREENSHOT_URLS)
+            if screenshot_url and len(screenshot_url.split(",")) > 1:
+                for screenshot in screenshot_url.split(","):
+                    screenshot_list.append({
+                        'small_image_url': screenshot,
+                        'large_image_url': screenshot,
+                        'version': self.version,
+                    })
+        return screenshot_list
+
+    def query_multiple_screenshots(self):
+        """ query if multiple screenshots for the given app are available
+            and if so, emit "screenshots-available" signal
+        """
+        # get screenshot list from the db, if that is empty thats fine,
+        # and we will query the screenshot server
+        if not self._screenshot_list:
+            self._screenshot_list = self._get_multiple_screenshots_from_db()
+        # check if we have it cached
+        if self._screenshot_list:
+            self.emit("screenshots-available", self._screenshot_list)
+            return
+        # download it
+        distro = self._distro
+        url = distro.SCREENSHOT_JSON_URL % self._app.pkgname
+        try:
+            f = Gio.File.new_for_uri(url)
+            f.load_contents_async(
+                None, self._gio_screenshots_json_download_complete_cb, None)
+        except:
+            LOG.exception("failed to load content")
+
+    def _sort_screenshots_by_best_version(self, screenshot_list):
+        """ take a screenshot result dict from screenshots.debian.org
+            and sort it
+        """
+        from softwarecenter.utils import version_compare
+        my_version = self.version
+        # discard screenshots which are more recent than the available version
+        for item in screenshot_list[:]:
+            v = item['version']
+            if v and version_compare(my_version, v) < 0:
+                screenshot_list.remove(item)
+        # now sort from high to low
+        return sorted(
+            screenshot_list,
+            cmp=lambda a, b: version_compare(a["version"] or '',
+                                             b["version"] or ''),
+            reverse=True)
+
+    def _gio_screenshots_json_download_complete_cb(self, source, result, path):
+        try:
+            res, content, etag = source.load_contents_finish(result)
+        except GObject.GError:
+            # ignore read errors, most likely transient
+            return
+        if content is not None:
+            try:
+                content = json.loads(content)
+            except ValueError as e:
+                LOG.error("can not decode: '%s' (%s)" % (content, e))
+                content = None
+
+        if isinstance(content, dict):
+            # a list of screenshots as listsed online
+            screenshot_list = content['screenshots']
+        else:
+            # fallback to a list of screenshots as supplied by the axi
+            screenshot_list = []
+
+        # save for later and emit
+        self._screenshot_list = self._sort_screenshots_by_best_version(
+            screenshot_list)
+        self.emit("screenshots-available", self._screenshot_list)
 
     @property
     def summary(self):
+        # not-automatic
+        if self._pkg and self._app.archive_suite:
+            ver = self._get_version_for_archive_suite(
+                self._pkg, self._app.archive_suite)
+            return ver.summary
+        # normal case
         if self._doc:
-            return self._db.get_summary(self._doc)
+            return capitalize_first_word(self._db.get_summary(self._doc))
         elif self._pkg:
             return self._pkg.candidate.summary
 
@@ -499,23 +712,88 @@ class AppDetails(object):
             if self._doc.get_value(XapianValues.THUMBNAIL_URL):
                 return self._doc.get_value(XapianValues.THUMBNAIL_URL)
         # else use the default
-        return self._distro.SCREENSHOT_THUMB_URL % { 'pkgname' : self.pkgname, 
-                                                     'version' : self.version or 0}
+        return self._distro.SCREENSHOT_THUMB_URL % {
+            'pkgname': self.pkgname,
+            'version': self.version or 0,
+        }
+
+    @property
+    def video_url(self):
+        # if there is a custom video url provided, use that
+        if self._doc:
+            if self._doc.get_value(XapianValues.VIDEO_URL):
+                return self._doc.get_value(XapianValues.VIDEO_URL)
+        # else use the video server
+        #return self._distro.VIDEO_URL % {
+        #    'pkgname' : self.pkgname,
+        #    'version' : self.version or 0,
+        #}
 
     @property
     def version(self):
         if self._pkg:
-            if self._pkg.installed:
+            if self._pkg.installed and not self._app.archive_suite:
                 return self._pkg.installed.version
             else:
-                return self._pkg.candidate.version
+                ver = self._get_version_for_archive_suite(
+                    self._pkg, self._app.archive_suite)
+                if ver:
+                    return ver.version
+        if self._doc:
+            return self._doc.get_value(XapianValues.VERSION_INFO)
+        LOG.warn("no version information found for '%s'" % self.pkgname)
+        return ""
+
+    def get_not_automatic_archive_versions(self):
+        """ this will return list of tuples (version, archive_suites)
+            with additional versions of the given package that can
+            be forced with force_not_automatic_archive_suite
+        """
+        archive_suites = []
+        if self._pkg:
+            for v in self._pkg.versions:
+                if v.not_automatic:
+                    archive_suites.append((v.version, v.origins[0].archive))
+        # if we have a not automatic version, ensure that the user can
+        # always pick the default too
+        if archive_suites:
+            # get candidate
+            ver = self._pkg.candidate
+            # if the candidate is the not-automatic version, find the first
+            # non-not-automatic one
+            if ver.not_automatic:
+                for ver in sorted(self._pkg.versions, reverse=True):
+                    if ver.downloadable and not ver.not_automatic:
+                        break
+            archive_suites.insert(0, (ver.version, ver.origins[0].archive))
+        return archive_suites
+
+    def force_not_automatic_archive_suite(self, archive_suite):
+        """ this will force to use the given "archive_suite" version
+            of the app (or clears it if archive_suite is empty)
+        """
+        # set or reset value
+        if archive_suite:
+            # add not-automatic suite to app
+            for ver in self._pkg.versions:
+                if archive_suite in [origin.archive for origin in ver.origins]:
+                    self._app.archive_suite = archive_suite
+                    return True
+            # no suitable archive found
+            raise ValueError("Pkg '%s' has no archive_suite '%s'" % (
+                    self._pkg, archive_suite))
+        else:
+            # clear version
+            self._app.archive_suite = ""
+            return True
 
     @property
     def warning(self):
         # apturl minver matches
         if not self.pkg_state == PkgStates.INSTALLED:
             if self._app.request:
-                minver_matches = re.findall(r'minver=[a-z,0-9,-,+,.,~]*', self._app.request)
+                minver_matches = re.findall(r'minver=[a-z,0-9,-,+,.,~]*',
+                    self._app.request)
                 if minver_matches and self.version:
                     minver = minver_matches[0][7:]
                     from softwarecenter.utils import version_compare
@@ -526,18 +804,19 @@ class AppDetails(object):
             source_to_enable = None
             if self.channelname and self._unavailable_channel():
                 source_to_enable = self.channelname
-            elif (self.component and 
+            elif (self.component and
                   self.component not in ("independent", "commercial")):
                 source_to_enable = self.component
             if source_to_enable:
                 sources = source_to_enable.split('&')
                 sources_length = len(sources)
                 if sources_length == 1:
-                    warning = _("Available from the \"%s\" source.") % sources[0]
+                    warning = (_("Available from the \"%s\" source.") %
+                        sources[0])
                 elif sources_length > 1:
-                    # Translators: the visible string is constructed concatenating 
-                    # the following 3 strings like this: 
-                    # Available from the following sources: %s, ... %s, %s.                
+                    # Translators: the visible string is constructed
+                    # concatenating the following 3 strings like this:
+                    # Available from the following sources: %s, ... %s, %s.
                     warning = _("Available from the following sources: ")
                     # Cycle through all, but the last
                     for source in sources[:-1]:
@@ -549,29 +828,71 @@ class AppDetails(object):
     def website(self):
         if self._pkg:
             return self._pkg.website
-    
+
+    @property
+    def supportsite(self):
+        if self._doc:
+            return self._doc.get_value(XapianValues.SUPPORT_SITE_URL)
+
     @property
     def license_key(self):
         if self._doc:
             return self._doc.get_value(XapianValues.LICENSE_KEY)
+        return ""
 
     @property
     def license_key_path(self):
         if self._doc:
             return self._doc.get_value(XapianValues.LICENSE_KEY_PATH)
 
+    @property
+    def region_requirements_satisfied(self):
+        my_region = get_region_cached()["countrycode"]
+        # if there are no region tag we are good
+        res = True
+        for tag in self.tags:
+            if tag.startswith(REGIONTAG):
+                # we found a region tag, now the region must match
+                res = False
+            if tag == REGIONTAG + my_region:
+                # we have the right region
+                return True
+        return res
+
+    @property
+    def hardware_requirements_satisfied(self):
+        for key, value in self.hardware_requirements.iteritems():
+            if value == "no":
+                return False
+        return True
+
+    @property
+    def hardware_requirements(self):
+        result = {}
+        try:
+            from softwarecenter.hw import get_hardware_support_for_tags
+            result = get_hardware_support_for_tags(self.tags)
+        except ImportError:
+            LOG.warn("failed to import debtagshw")
+            return result
+        return result
+
     def _unavailable_channel(self):
-        """ Check if the given doc refers to a channel that is currently not enabled """
+        """ Check if the given doc refers to a channel that is currently
+            not enabled
+        """
         return not is_channel_available(self.channelname)
 
     def _unavailable_component(self, component_to_check=None):
-        """ Check if the given doc refers to a component that is currently not enabled """
+        """ Check if the given doc refers to a component that is currently
+            not enabled
+        """
         if component_to_check:
             component = component_to_check
         elif self.component:
             component = self.component
         else:
-            component =  self._doc.get_value(XapianValues.ARCHIVE_SECTION)
+            component = self._doc.get_value(XapianValues.ARCHIVE_SECTION)
         if not component:
             return False
         distro_codename = self._distro.get_codename()
@@ -598,6 +919,9 @@ class AppDetails(object):
         details.append("   installation_date: %s" % self.installation_date)
         details.append("       purchase_date: %s" % self.purchase_date)
         details.append("             license: %s" % self.license)
+        details.append("         license_key: %s" % self.license_key[0:3] +
+            len(self.license_key) * "*")
+        details.append("    license_key_path: %s" % self.license_key_path)
         details.append("      date_published: %s" % self.date_published)
         details.append("  maintenance_status: %s" % self.maintenance_status)
         details.append("           pkg_state: %s" % self.pkg_state)
