@@ -234,6 +234,7 @@ class AptCache(PackageInfo):
     def open(self):
         """ (re)open the cache, this sends cache-invalid, cache-ready signals
         """
+        LOG.info("aptcache.open()")
         self._ready = False
         self.emit("cache-invalid")
         from softwarecenter.utils import ExecutionTime
@@ -596,6 +597,10 @@ class AptCache(PackageInfo):
                 self._set_candidate_release(pkg, archive_suite)
             # now get the right version
             version = self._cache[p].candidate
+            # this can happen on e.g. deb packages that are not in the cache
+            # testcase: software-center google-chrome-stable_current_amd64.deb
+            if not version:
+                continue
             pkgs_to_install.append(version)
             # now do it
             deps_inst = self._try_install_and_get_all_deps_installed(
