@@ -23,6 +23,10 @@ import logging.handlers
 import os.path
 
 from paths import SOFTWARE_CENTER_CACHE_DIR
+from utils import (
+    ensure_file_writable_and_delete_if_not,
+    safe_makedirs,
+)
 
 """ setup global logging for software-center """
 
@@ -92,8 +96,7 @@ root.addHandler(handler)
 handler.addFilter(NullFilterThatWarnsAboutRootLoggerUsage())
 
 # create log file
-if not os.path.exists(SOFTWARE_CENTER_CACHE_DIR):
-    os.makedirs(SOFTWARE_CENTER_CACHE_DIR)
+safe_makedirs(SOFTWARE_CENTER_CACHE_DIR)
 logfile_path = os.path.join(SOFTWARE_CENTER_CACHE_DIR, "software-center.log")
 
 # try to fix inaccessible s-c directory (#688682)
@@ -107,15 +110,10 @@ if not os.access(SOFTWARE_CENTER_CACHE_DIR, os.W_OK):
         if not os.path.exists(target):
             os.rename(SOFTWARE_CENTER_CACHE_DIR, target)
             break
-    os.makedirs(SOFTWARE_CENTER_CACHE_DIR)
+    safe_makedirs(SOFTWARE_CENTER_CACHE_DIR)
 
 # according to bug 688682 many people have a non-writeable logfile
-if os.path.exists(logfile_path) and not os.access(logfile_path, os.W_OK):
-    try:
-        logging.warn("trying to fix non-writeable logfile")
-        os.remove(logfile_path)
-    except:
-        logging.exception("failed to fix non-writeable logfile")
+ensure_file_writable_and_delete_if_not(logfile_path)
 
 logfile_handler = logging.handlers.RotatingFileHandler(
     logfile_path, maxBytes=100 * 1000, backupCount=5)
