@@ -21,7 +21,9 @@ from gi.repository import GObject
 from gi.repository import Gtk
 import logging
 import xapian
+import os
 
+import softwarecenter.utils
 import softwarecenter.ui.gtk3.dialogs as dialogs
 from softwarecenter.ui.gtk3.models.appstore2 import AppListStore
 
@@ -30,28 +32,45 @@ from gettext import gettext as _
 from softwarecenter.enums import (ActionButtons,
                                   NavButtons,
                                   NonAppVisibility,
-                                  DEFAULT_SEARCH_LIMIT)
+                                  DEFAULT_SEARCH_LIMIT,
+                                  TransactionTypes)
 from softwarecenter.paths import APP_INSTALL_PATH
-from softwarecenter.utils import wait_for_apt_cache_ready
+from softwarecenter.utils import (wait_for_apt_cache_ready,
+                                  get_exec_line_from_desktop,
+                                  is_no_display_desktop_file,
+                                  get_file_path_from_iconname,
+                                  convert_desktop_file_to_installed_location)
 from softwarecenter.db.appfilter import AppFilter
 from softwarecenter.db.database import Application
+
 from softwarecenter.ui.gtk3.views.purchaseview import PurchaseView
 
 from softwarecenter.ui.gtk3.views.catview_gtk import (LobbyViewGtk,
                                                       SubCategoryViewGtk)
 from softwarepane import SoftwarePane
 from softwarecenter.ui.gtk3.session.viewmanager import get_viewmanager
+from softwarecenter.ui.gtk3.session.appmanager import get_appmanager
+from softwarecenter.backend.channel import SoftwareChannel
+from softwarecenter.backend.unitylauncher import (UnityLauncher,
+                                                  UnityLauncherInfo,
+                                                  TransactionDetails)
 
 LOG = logging.getLogger(__name__)
 
+
 class AvailablePane(SoftwarePane):
     """Widget that represents the available panel in software-center
        It contains a search entry and navigation buttons
     """
 
     class Pages():
-        # page names, useful for debuggin
-        NAMES = ('lobby', 'subcategory', 'list', 'details', 'purchase')
+        # page names, useful for debugging
+        NAMES = ('lobby',
+                 'subcategory',
+                 'list',
+                 'details',
+                 'purchase',
+                )
         # actual page id's
         (LOBBY,
          SUBCATEGORY,
@@ -61,17 +80,17 @@ class AvailablePane(SoftwarePane):
         # the default page
         HOME = LOBBY
 
-    __gsignals__ = {'available-pane-created':(GObject.SignalFlags.RUN_FIRST,
-                                              None,
-                                              ())}
+    __gsignals__ = {'available-pane-created': (GObject.SignalFlags.RUN_FIRST,
+                                               None,
+                                               ())}
 
-    def __init__(self, 
+    def __init__(self,
                  cache,
-                 db, 
-                 distro, 
-                 icons, 
-                 datadir, 
-                 navhistory_back_action, 
+                 db,
+                 distro,
+                 icons,
+                 datadir,
+                 navhistory_back_action,
                  navhistory_forward_action):
         # parent
         SoftwarePane.__init__(self, cache, db, distro, icons, datadir)
@@ -86,16 +105,27 @@ class AvailablePane(SoftwarePane):
         self.current_app_by_category = {}
         self.current_app_by_subcategory = {}
         self.pane_name = _("Get Software")
-        
+
         # views to be created in init_view
         self.cat_view = None
         self.subcategories_view = None
 
+        # integrate with the Unity launcher
+        self.unity_launcher = UnityLauncher()
+        # keep track of applications that are queued to be added
+        # to the Unity launcher
+        self.unity_launcher_transaction_queue = {}
+        # flag to indicate whether applications should be added to the
+        # unity launcher when installed (this value is initialized by
+        # the config load in app.py)
+        self.add_to_launcher_enabled = True
+
     def init_view(self):
-        if self.view_initialized: 
+        if self.view_initialized:
             return
 
         self.show_appview_spinner()
+
         window = self.get_window()
         if window is not None:
             window.set_cursor(self.busy_cursor)
@@ -111,20 +141,28 @@ class AvailablePane(SoftwarePane):
             #~ self.app_view._append_appcount(appcount)
         #~ liststore.connect('appcount-changed', on_appcount_changed)
         self.app_view.set_model(liststore)
-        # setup purchase stuff
-        self.app_details_view.connect("purchase-requested",
-                                      self.on_purchase_requested)
-        liststore.connect(
-            "needs-refresh", lambda helper, pkgname: self.app_view.queue_draw())
+        liststore.connect("needs-refresh",
+            lambda helper, pkgname: self.app_view.queue_draw())
 
         # purchase view
         self.purchase_view = PurchaseView()
-        self.purchase_view.connect("purchase-succeeded", self.on_purchase_succeeded)
-        self.purchase_view.connect("purchase-failed", self.on_purchase_failed)
-        self.purchase_view.connect("purchase-cancelled-by-user", self.on_purchase_cancelled_by_user)
+        app_manager = get_appmanager()
+        app_manager.connect("purchase-requested",
+            self.on_purchase_requested)
+        self.purchase_view.connect("purchase-succeeded",
+            self.on_purchase_succeeded)
+        self.purchase_view.connect("purchase-failed",
+            self.on_purchase_failed)
+        self.purchase_view.connect("purchase-cancelled-by-user",
+            self.on_purchase_cancelled_by_user)
+        self.purchase_view.connect("terms-of-service-declined",
+            self.on_terms_of_service_declined)
+        self.purchase_view.connect("purchase-needs-spinner",
+            self.on_purchase_needs_spinner)
+
         # categories, appview and details into the notebook in the bottom
         self.scroll_categories = Gtk.ScrolledWindow()
-        self.scroll_categories.set_policy(Gtk.PolicyType.AUTOMATIC, 
+        self.scroll_categories.set_policy(Gtk.PolicyType.AUTOMATIC,
                                         Gtk.PolicyType.AUTOMATIC)
         self.cat_view = LobbyViewGtk(self.datadir, APP_INSTALL_PATH,
                                        self.cache,
@@ -132,16 +170,17 @@ class AvailablePane(SoftwarePane):
                                        self.icons,
                                        self.apps_filter)
         self.scroll_categories.add(self.cat_view)
-        self.notebook.append_page(self.scroll_categories, Gtk.Label(label="categories"))
+        self.notebook.append_page(self.scroll_categories,
+            Gtk.Label(label="categories"))
 
         # sub-categories view
         self.subcategories_view = SubCategoryViewGtk(self.datadir,
-                                                 APP_INSTALL_PATH,
-                                                 self.cache,
-                                                 self.db,
-                                                 self.icons,
-                                                 self.apps_filter,
-                                                 root_category=self.cat_view.categories[0])
+            APP_INSTALL_PATH,
+            self.cache,
+            self.db,
+            self.icons,
+            self.apps_filter,
+            root_category=self.cat_view.categories[0])
         self.subcategories_view.connect(
             "category-selected", self.on_subcategory_activated)
         self.subcategories_view.connect(
@@ -172,13 +211,23 @@ class AvailablePane(SoftwarePane):
             "application-activated", self.on_application_activated)
 
         # details
-        self.notebook.append_page(self.scroll_details, Gtk.Label(label=NavButtons.DETAILS))
+        self.notebook.append_page(self.scroll_details,
+            Gtk.Label(label=NavButtons.DETAILS))
 
         # purchase view
-        self.notebook.append_page(self.purchase_view, Gtk.Label(label=NavButtons.PURCHASE))
+        self.notebook.append_page(self.purchase_view,
+            Gtk.Label(label=NavButtons.PURCHASE))
 
         # install backend
-        self.backend.connect("transactions-changed", self._on_transactions_changed)
+        self.backend.connect("transaction-started",
+            self.on_transaction_started)
+        self.backend.connect("transactions-changed",
+            self.on_transactions_changed)
+        self.backend.connect("transaction-finished",
+            self.on_transaction_complete)
+        self.backend.connect("transaction-stopped",
+            self.on_transaction_complete)
+
         # now we are initialized
         self.searchentry.set_sensitive(True)
         self.emit("available-pane-created")
@@ -190,6 +239,7 @@ class AvailablePane(SoftwarePane):
         # again (LP: #851671)
         self.view_initialized = True
 
+        # important to "seed" the initial history stack (LP: #1005104)
         vm = get_viewmanager()
         vm.display_page(
             self, AvailablePane.Pages.LOBBY,
@@ -198,28 +248,40 @@ class AvailablePane(SoftwarePane):
         if window is not None:
             window.set_cursor(None)
 
-    def on_purchase_requested(self, widget, app, url):
+    def on_purchase_requested(self, appmanager, app, iconname, url):
+        if self.purchase_view.initiate_purchase(app, iconname, url):
+            vm = get_viewmanager()
+            vm.display_page(
+                self, AvailablePane.Pages.PURCHASE, self.state,
+                self.display_purchase)
 
-        self.appdetails = app.get_details(self.db)
-        iconname = self.appdetails.icon
-        self.purchase_view.initiate_purchase(app, iconname, url)
+    def on_purchase_needs_spinner(self, appmanager, active):
         vm = get_viewmanager()
-        vm.display_page(self, AvailablePane.Pages.PURCHASE, self.state, self.display_purchase)
-        
+        vm.set_spinner_active(active)
+
     def on_purchase_succeeded(self, widget):
         # switch to the details page to display the transaction is in progress
         self._return_to_appdetails_view()
-        
+
     def on_purchase_failed(self, widget):
         self._return_to_appdetails_view()
         dialogs.error(None,
                       _("Failure in the purchase process."),
                       _("Sorry, something went wrong. Your payment "
                         "has been cancelled."))
-        
+
     def on_purchase_cancelled_by_user(self, widget):
         self._return_to_appdetails_view()
 
+    def on_terms_of_service_declined(self, widget):
+        """ The Terms of Service dialog was declined by the user, so we just
+            reset the purchase button in case they want another chance
+        """
+        if self.is_app_details_view_showing():
+            self.app_details_view.pkg_statusbar.button.set_sensitive(True)
+        elif self.is_applist_view_showing():
+            self.app_view.tree_view.reset_action_button()
+
     def _return_to_appdetails_view(self):
         vm = get_viewmanager()
         vm.nav_back()
@@ -277,15 +339,16 @@ class AvailablePane(SoftwarePane):
     #~ def _show_hide_subcategories(self, show_category_applist=False):
         #~ # check if have subcategories and are not in a subcategory
         #~ # view - if so, show it
-        #~ if (self.notebook.get_current_page() == AvailablePane.Pages.LOBBY or
-            #~ self.notebook.get_current_page() == AvailablePane.Pages.DETAILS):
+        #~ current_page = self.notebook.get_current_page()
+        #~ if (current_page == AvailablePane.Pages.LOBBY or
+            #~ current_page == AvailablePane.Pages.DETAILS):
             #~ return
         #~ if (not show_category_applist and
             #~ self.state.category and
             #~ self.state.category.subcategories and
             #~ not (self.state.search_term or self.state.subcategory)):
             #~ self.subcategories_view.set_subcategory(self.state.category,
-                                                    #~ num_items=len(self.app_view.get_model()))
+                #~ num_items=len(self.app_view.get_model()))
             #~ self.notebook.set_current_page(AvailablePane.Pages.SUBCATEGORY)
         #~ else:
             #~ self.notebook.set_current_page(AvailablePane.Pages.LIST)
@@ -296,7 +359,8 @@ class AvailablePane(SoftwarePane):
             return None
         else:
             if self.state.subcategory:
-                return self.current_app_by_subcategory.get(self.state.subcategory)
+                return self.current_app_by_subcategory.get(
+                    self.state.subcategory)
             else:
                 return self.current_app_by_category.get(self.state.category)
 
@@ -306,23 +370,117 @@ class AvailablePane(SoftwarePane):
             return self.state.subcategory
         elif self.state.category:
             return self.state.category
-        return None
 
     def unset_current_category(self):
-        """ unset the current showing category, but keep e.g. the current 
-            search 
+        """ unset the current showing category, but keep e.g. the current
+            search
         """
-
         self.state.category = None
         self.state.subcategory = None
+        # reset the non-global filters see (LP: #985389)
+        if self.state.filter:
+            self.state.filter.reset()
 
-    def _on_transactions_changed(self, *args):
+    def on_transaction_started(self, backend, pkgname, appname, trans_id,
+                               trans_type):
+        self._register_unity_launcher_transaction_started(pkgname, appname,
+                                                          trans_id, trans_type)
+
+    def on_transactions_changed(self, backend, pending_transactions):
         """internal helper that keeps the action bar up-to-date by
            keeping track of the transaction-started signals
         """
         if self._is_custom_list_search(self.state.search_term):
             self._update_action_bar()
 
+    def on_transaction_complete(self, backend, result):
+        if result.pkgname in self.unity_launcher_transaction_queue:
+            transaction_details = (
+                self.unity_launcher_transaction_queue.pop(result.pkgname))
+            self._add_application_to_unity_launcher(transaction_details)
+
+    def _register_unity_launcher_transaction_started(self, pkgname, appname,
+                                                     trans_id, trans_type):
+        # at the start of the transaction, we gather details for use later
+        # when it is time to add the installed app to the Unity launcher,
+        # (see #972710). we only care about installs for the launcher
+        # mvo: use softwarecenter.utils explicitly here so that we can monkey
+        #      patch it in the test
+        if (trans_type == TransactionTypes.INSTALL and
+                self.add_to_launcher_enabled and
+                softwarecenter.utils.is_unity_running()):
+            transaction_details = TransactionDetails(
+                    pkgname, appname, trans_id, trans_type)
+            self.unity_launcher_transaction_queue[pkgname] = (
+                    transaction_details)
+
+    def _add_application_to_unity_launcher(self, transaction_details):
+        app = Application(pkgname=transaction_details.pkgname,
+                          appname=transaction_details.appname)
+        appdetails = app.get_details(self.db)
+        # convert the app-install desktop file location to the actual installed
+        # desktop file location (or in the case of a purchased item from the
+        # agent, generate the correct installed desktop file location)
+        installed_desktop_file_path = (
+            convert_desktop_file_to_installed_location(appdetails.desktop_file,
+                                                       app.pkgname))
+        # we only add items to the launcher that have a desktop file
+        if not installed_desktop_file_path:
+            return
+        # do not add apps that have no Exec entry in their desktop file
+        # (e.g. wine, see LP: #848437 or ubuntu-restricted-extras,
+        # see LP: #913756), also, don't add the item if NoDisplay is
+        # specified (see LP: #1006483)
+        if (os.path.exists(installed_desktop_file_path) and
+            (not get_exec_line_from_desktop(installed_desktop_file_path) or
+            is_no_display_desktop_file(installed_desktop_file_path))):
+            return
+
+        # now gather up the unity launcher info items and send the app to the
+        # launcher service
+        launcher_info = self._get_unity_launcher_info(app, appdetails,
+                installed_desktop_file_path,
+                transaction_details.trans_id)
+        self.unity_launcher.send_application_to_launcher(
+                transaction_details.pkgname,
+                launcher_info)
+
+    def _get_unity_launcher_info(
+            self, app, appdetails, installed_desktop_file_path, trans_id):
+        (icon_size, icon_x, icon_y) = (
+                self._get_onscreen_icon_details_for_launcher_service(app))
+        icon_path = get_file_path_from_iconname(
+                                self.icons,
+                                iconname=appdetails.icon)
+        # Note that the transaction ID is not specified because we are firing
+        # the dbus signal at the very end of the installation now, and the
+        # corresponding fix in Unity is expecting this value to be empty (it
+        # would not be useful to the Unity launcher at this point anyway,
+        # as the corresponding transaction would be complete).
+        # Please see bug LP: #925014 and corresponding Unity branch for
+        # further details.
+        # TODO: If and when we re-implement firing the dbus signal at the
+        # start of the transaction, at that time we will need to replace
+        # the empty string below with the trans_id value.
+        launcher_info = UnityLauncherInfo(app.name,
+                                          appdetails.icon,
+                                          icon_path,
+                                          icon_x,
+                                          icon_y,
+                                          icon_size,
+                                          installed_desktop_file_path,
+                                          "")
+        return launcher_info
+
+    def _get_onscreen_icon_details_for_launcher_service(self, app):
+        if self.is_app_details_view_showing():
+            return self.app_details_view.get_app_icon_details()
+        elif self.is_applist_view_showing():
+            return self.app_view.get_app_icon_details()
+        else:
+            # set a default, even though we cannot install from the other panes
+            return (0, 0, 0)
+
     def on_app_list_changed(self, pane, length):
         """internal helper that keeps the status text and the action
            bar up-to-date by keeping track of the app-list-changed
@@ -336,10 +494,10 @@ class AvailablePane(SoftwarePane):
         self._update_action_bar_buttons()
 
     def _update_action_bar_buttons(self):
-        '''
-        update buttons in the action bar to implement the custom package lists feature,
-        see https://wiki.ubuntu.com/SoftwareCenter#Custom%20package%20lists
-        '''
+        """Update buttons in the action bar to implement the custom package
+           lists feature, see
+           https://wiki.ubuntu.com/SoftwareCenter#Custom%20package%20lists
+        """
         if self._is_custom_list_search(self.state.search_term):
             installable = []
             for doc in self.enquirer.get_documents():
@@ -349,9 +507,10 @@ class AvailablePane(SoftwarePane):
                     not len(self.backend.pending_transactions) > 0):
                     app = Application(pkgname=pkgname)
                     installable.append(app)
-            button_text = gettext.ngettext("Install %(amount)s Item",
-                                           "Install %(amount)s Items",
-                                            len(installable)) % { 'amount': len(installable), }
+            button_text = gettext.ngettext(
+                "Install %(amount)s Item",
+                "Install %(amount)s Items",
+                len(installable)) % {'amount': len(installable)}
             button = self.action_bar.get_button(ActionButtons.INSTALL)
             if button and installable:
                 # Install all already offered. Update offer.
@@ -367,13 +526,12 @@ class AvailablePane(SoftwarePane):
         else:
             # Ensure button is removed.
             self.action_bar.remove_button(ActionButtons.INSTALL)
-            
+
     def _install_current_appstore(self):
         '''
         Function that installs all applications displayed in the pane.
         '''
-        pkgnames = []
-        appnames = []
+        apps = []
         iconnames = []
         self.action_bar.remove_button(ActionButtons.INSTALL)
         for doc in self.enquirer.get_documents():
@@ -381,24 +539,31 @@ class AvailablePane(SoftwarePane):
             if (pkgname in self.cache and
                 not self.cache[pkgname].is_installed and
                 pkgname not in self.backend.pending_transactions):
-                pkgnames.append(pkgname)
-                appnames.append(self.db.get_appname(doc))
+                apps.append(self.db.get_application(doc))
                 # add iconnames
                 iconnames.append(self.db.get_iconname(doc))
-        self.backend.install_multiple(pkgnames, appnames, iconnames)
+        self.backend.install_multiple(apps, iconnames)
+
+    def _show_or_hide_search_combo_box(self, view_state):
+        # show/hide the sort combobox headers if the category forces a
+        # custom sort mode
+        category = view_state.category
+        allow_user_sort = category is None or not category.is_forced_sort_mode
+        self.app_view.set_allow_user_sorting(allow_user_sort)
 
     def set_state(self, nav_item):
-        return
+        pass
 
     def _clear_search(self):
         self.searchentry.clear_with_no_signal()
         self.apps_limit = 0
         self.apps_search_term = ""
-        
+        self.state.search_term = ""
+
     def _is_custom_list_search(self, search_term):
-        return (search_term and 
+        return (search_term and
                 ',' in search_term)
-        
+
     # callbacks
     def on_cache_ready(self, cache):
         """ refresh the application list when the cache is re-opened """
@@ -411,13 +576,20 @@ class AvailablePane(SoftwarePane):
         """callback when the search entry widget changes"""
         LOG.debug("on_search_terms_changed: %s" % new_text)
 
+        # reset the flag in the app_view because each new search should
+        # reset the sort criteria
+        self.app_view.reset_default_sort_mode()
+
         self.state.search_term = new_text
-        
+
         # do not hide technical items for a custom list search
         if self._is_custom_list_search(self.state.search_term):
             self.nonapps_visible = NonAppVisibility.ALWAYS_VISIBLE
 
         vm = get_viewmanager()
+        adj = self.app_view.tree_view_scroll.get_vadjustment()
+        if adj:
+            adj.set_value(0.0)
 
         # yeah for special cases - as discussed on irc, mpt
         # wants this to return to the category screen *if*
@@ -436,7 +608,7 @@ class AvailablePane(SoftwarePane):
                             self.display_app_view_page)
             return False
 
-        elif (self.state.category and 
+        elif (self.state.category and
                 self.state.category.subcategories and not new_text):
             vm.display_page(self,
                            AvailablePane.Pages.SUBCATEGORY,
@@ -447,8 +619,8 @@ class AvailablePane(SoftwarePane):
                         self.display_search_page)
 
     def on_db_reopen(self, db):
-        " called when the database is reopened"
-        #print "on_db_open"
+        """Called when the database is reopened."""
+        super(AvailablePane, self).on_db_reopen(db)
         self.refresh_apps()
         if self.app_details_view:
             self.app_details_view.refresh_app()
@@ -489,6 +661,9 @@ class AvailablePane(SoftwarePane):
 
         header_strings = self._get_header_for_view_state(view_state)
         self.app_view.set_header_labels(*header_strings)
+        self._show_or_hide_search_combo_box(view_state)
+
+        self.app_view.vadj = view_state.vadjustment
 
         self.refresh_apps()
         self.cat_view.stop_carousels()
@@ -502,7 +677,7 @@ class AvailablePane(SoftwarePane):
             self.refresh_apps()
 
         query = self.get_query()
-        n_matches = self.quick_query(query)
+        n_matches = self.quick_query_len(query)
         self.subcategories_view.set_subcategory(category, n_matches)
 
         self.action_bar.clear()
@@ -515,14 +690,13 @@ class AvailablePane(SoftwarePane):
 
         header_strings = self._get_header_for_view_state(view_state)
         self.app_view.set_header_labels(*header_strings)
-        # hide the sort combobox headers if the category forces a
-        # custom sort mode
-        allow_user_sort = category is None or not category.is_forced_sort_mode
-        self.app_view.set_allow_user_sorting(allow_user_sort)
+        self._show_or_hide_search_combo_box(view_state)
 
         if view_state.search_term:
             self._clear_search()
 
+        self.app_view.vadj = view_state.vadjustment
+
         self.refresh_apps()
         self.cat_view.stop_carousels()
         return True
@@ -537,23 +711,23 @@ class AvailablePane(SoftwarePane):
         SoftwarePane.display_details_page(self, page, view_state)
         self.cat_view.stop_carousels()
         return True
-        
+
     def display_purchase(self, page, view_state):
         self.notebook.set_current_page(AvailablePane.Pages.PURCHASE)
         self.action_bar.clear()
         self.cat_view.stop_carousels()
-        return
-        
+
     def display_previous_purchases(self, page, view_state):
         self.nonapps_visible = NonAppVisibility.ALWAYS_VISIBLE
-        self.app_view.set_header_labels(_("Previous Purchases"), None)
+        header_strings = self._get_header_for_view_state(view_state)
+        self.app_view.set_header_labels(*header_strings)
         self.notebook.set_current_page(AvailablePane.Pages.LIST)
-        # do not emit app-list-changed here, this is done async when
-        # the new model is ready
-        self.refresh_apps(query=self.previous_purchases_query)
+        # clear any search terms
+        self._clear_search()
+        self.refresh_apps()
         self.action_bar.clear()
         self.cat_view.stop_carousels()
-        return
+        return True
 
     def on_subcategory_activated(self, subcat_view, category):
         LOG.debug("on_subcategory_activated: %s %s" % (
@@ -604,12 +778,14 @@ class AvailablePane(SoftwarePane):
 
     def on_show_category_applist(self, widget):
         self._show_hide_subcategories(show_category_applist=True)
-        
+
     def on_previous_purchases_activated(self, query):
         """ called to activate the previous purchases view """
         #print cat_view, name, query
         LOG.debug("on_previous_purchases_activated with query: %s" % query)
-        self.previous_purchases_query = query
+        self.state.channel = SoftwareChannel("Previous Purchases",
+                                             "software-center-agent",
+                                             None, channel_query=query)
         vm = get_viewmanager()
         vm.display_page(self, AvailablePane.Pages.LIST, self.state,
                         self.display_previous_purchases)
@@ -618,13 +794,14 @@ class AvailablePane(SoftwarePane):
         """ Return True if we are in the category page or if we display a
             sub-category page
         """
-        return (self.notebook.get_current_page() == AvailablePane.Pages.LOBBY or \
-                self.notebook.get_current_page() == AvailablePane.Pages.SUBCATEGORY)
+        current_page = self.notebook.get_current_page()
+        return (current_page == AvailablePane.Pages.LOBBY or
+            current_page == AvailablePane.Pages.SUBCATEGORY)
 
     def is_applist_view_showing(self):
         """Return True if we are in the applist view """
         return self.notebook.get_current_page() == AvailablePane.Pages.LIST
-        
+
     def is_app_details_view_showing(self):
         """Return True if we are in the app_details view """
         return self.notebook.get_current_page() == AvailablePane.Pages.DETAILS
@@ -644,12 +821,14 @@ class AvailablePane(SoftwarePane):
         if not self.state.filter:
             self.state.filter = AppFilter(self.db, self.cache)
 
-        if category and category.flags and 'available-only' in category.flags:
+        if (category and category.flags and
+            'available-only' in category.flags):
             self.state.filter.set_available_only(True)
         else:
             self.state.filter.set_available_only(False)
 
-        if category and category.flags and 'not-installed-only' in category.flags:
+        if (category and category.flags and
+            'not-installed-only' in category.flags):
             self.state.filter.set_not_installed_only(True)
         else:
             self.state.filter.set_not_installed_only(False)
@@ -667,29 +846,38 @@ class AvailablePane(SoftwarePane):
 def get_test_window():
     from softwarecenter.testutils import (get_test_db,
                                           get_test_datadir,
+                                          get_test_install_backend,
                                           get_test_gtk3_viewmanager,
                                           get_test_pkg_info,
                                           get_test_gtk3_icon_cache,
                                           )
     # needed because available pane will try to get it
     vm = get_test_gtk3_viewmanager()
-    vm # make pyflakes happy
+    vm  # make pyflakes happy
     db = get_test_db()
     cache = get_test_pkg_info()
     datadir = get_test_datadir()
     icons = get_test_gtk3_icon_cache()
+    backend = get_test_install_backend()
 
-    navhistory_back_action = Gtk.Action("navhistory_back_action", "Back", "Back", None)
-    navhistory_forward_action = Gtk.Action("navhistory_forward_action", "Forward", "Forward", None)
+    # create global AppManager instance
+    from softwarecenter.ui.gtk3.session.appmanager import ApplicationManager
+    ApplicationManager(db, backend, icons)
 
-    w = AvailablePane(cache, db, 'Ubuntu', icons, datadir, navhistory_back_action, navhistory_forward_action)
+    navhistory_back_action = Gtk.Action("navhistory_back_action", "Back",
+        "Back", None)
+    navhistory_forward_action = Gtk.Action("navhistory_forward_action",
+        "Forward", "Forward", None)
+
+    w = AvailablePane(cache, db, 'Ubuntu', icons, datadir,
+        navhistory_back_action, navhistory_forward_action)
     w.init_view()
     w.show()
 
     win = Gtk.Window()
     win.connect("destroy", Gtk.main_quit)
     win.add(w)
-    win.set_size_request(800,600)
+    win.set_size_request(800, 600)
     win.show_all()
 
     # this is used later in tests
@@ -700,4 +888,3 @@ def get_test_window():
 if __name__ == "__main__":
     win = get_test_window()
     Gtk.main()
-
