@@ -30,20 +30,23 @@ import platform
 
 from softwarecenter.enums import (NonAppVisibility,
                                   SortMethods)
-from softwarecenter.utils import wait_for_apt_cache_ready
+from softwarecenter.utils import (
+    wait_for_apt_cache_ready, utf8, ExecutionTime)
 from softwarecenter.db.categories import (CategoriesParser,
                                           categories_sorted_by_name)
 from softwarecenter.ui.gtk3.models.appstore2 import (
     AppTreeStore, CategoryRowReference)
 from softwarecenter.ui.gtk3.widgets.menubutton import MenuButton
 from softwarecenter.ui.gtk3.widgets.oneconfviews import OneConfViews
-from softwarecenter.ui.gtk3.widgets.spinner import SpinnerView
+from softwarecenter.ui.gtk3.widgets.spinner import SpinnerNotebook
 from softwarecenter.ui.gtk3.views.appview import AppView
 from softwarepane import SoftwarePane
 from softwarecenter.backend.oneconfhandler import get_oneconf_handler
 from softwarecenter.db.appfilter import AppFilter
+from softwarecenter.paths import APP_INSTALL_PATH
+
+LOG = logging.getLogger(__name__)
 
-LOG=logging.getLogger(__name__)
 
 def interrupt_build_and_wait(f):
     """ decorator that ensures that a build of the categorised installed apps
@@ -77,19 +80,20 @@ class InstalledPane(SoftwarePane, CategoriesParser):
          DETAILS) = range(2)
         # the default page
         HOME = LIST
-        
+
     # pages for the installed view spinner notebook
     (PAGE_SPINNER,
      PAGE_INSTALLED) = range(2)
 
-    __gsignals__ = {'installed-pane-created':(GObject.SignalFlags.RUN_FIRST,
-                                              None,
-                                              ())}
+    __gsignals__ = {'installed-pane-created': (GObject.SignalFlags.RUN_FIRST,
+                                               None,
+                                               ())}
 
     def __init__(self, cache, db, distro, icons, datadir):
 
         # parent
-        SoftwarePane.__init__(self, cache, db, distro, icons, datadir, show_ratings=False)
+        SoftwarePane.__init__(self, cache, db, distro, icons, datadir,
+            show_ratings=False)
         CategoriesParser.__init__(self, db)
 
         self.current_appview_selection = None
@@ -109,32 +113,42 @@ class InstalledPane(SoftwarePane, CategoriesParser):
         self._halt_build = False
 
         self.nonapps_visible = NonAppVisibility.NEVER_VISIBLE
-        
+
         self.visible_docids = None
         self.visible_cats = {}
 
+        self.installed_spinner_notebook = None
+
     def init_view(self):
-        if self.view_initialized: return
+        if self.view_initialized:
+            return
 
         SoftwarePane.init_view(self)
-        
-        # show a busy cursor and display the main spinner while we build the view
+
+        # show a busy cursor and display the main spinner while we build the
+        # view
         window = self.get_window()
         if window:
             window.set_cursor(self.busy_cursor)
         self.show_appview_spinner()
-        
+
         self.oneconf_viewpickler = OneConfViews(self.icons)
-        self.oneconf_viewpickler.register_computer(None, _("This computer (%s)") % platform.node())
+        self.oneconf_viewpickler.register_computer(None,
+            _("This computer (%s)") % platform.node())
         self.oneconf_viewpickler.select_first()
-        self.oneconf_viewpickler.connect('computer-changed', self._selected_computer_changed)
-        self.oneconf_viewpickler.connect('current-inventory-refreshed', self._current_inventory_need_refresh)
-        
+        self.oneconf_viewpickler.connect('computer-changed',
+            self._selected_computer_changed)
+        self.oneconf_viewpickler.connect('current-inventory-refreshed',
+            self._current_inventory_need_refresh)
+
         # Start OneConf
         self.oneconf_handler = get_oneconf_handler(self.oneconf_viewpickler)
-        self.oneconf_handler.connect('show-oneconf-changed', self._show_oneconf_changed)
-        self.oneconf_handler.connect('last-time-sync-changed', self._last_time_sync_oneconf_changed)
-        
+        if self.oneconf_handler:
+            self.oneconf_handler.connect('show-oneconf-changed',
+                self._show_oneconf_changed)
+            self.oneconf_handler.connect('last-time-sync-changed',
+                self._last_time_sync_oneconf_changed)
+
         # OneConf pane
         self.computerpane = Gtk.Paned.new(Gtk.Orientation.HORIZONTAL)
         self.oneconfcontrol = Gtk.Box()
@@ -148,13 +162,18 @@ class InstalledPane(SoftwarePane, CategoriesParser):
         scroll.set_shadow_type(Gtk.ShadowType.IN)
         scroll.add(self.oneconf_viewpickler)
         self.oneconfcontrol.pack_start(scroll, True, True, 0)
-        
+
         oneconftoolbar = Gtk.Box()
         oneconftoolbar.set_orientation(Gtk.Orientation.HORIZONTAL)
         oneconfpropertymenu = Gtk.Menu()
-        self.oneconfproperty = MenuButton(oneconfpropertymenu, Gtk.Image.new_from_stock(Gtk.STOCK_PROPERTIES, Gtk.IconSize.BUTTON))
-        stop_oneconf_share_menuitem = Gtk.MenuItem(label=_(u"Stop Syncing “%s”") % platform.node())
-        stop_oneconf_share_menuitem.connect("activate", self._on_stop_showing_oneconf_clicked)
+        self.oneconfproperty = MenuButton(oneconfpropertymenu,
+            Gtk.Image.new_from_stock(Gtk.STOCK_PROPERTIES,
+            Gtk.IconSize.BUTTON))
+        self.stopsync_label = _(u"Stop Syncing “%s”")
+        stop_oneconf_share_menuitem = Gtk.MenuItem(
+            label=self.stopsync_label % platform.node())
+        stop_oneconf_share_menuitem.connect("activate",
+            self._on_stop_oneconf_hostshare_clicked)
         stop_oneconf_share_menuitem.show()
         oneconfpropertymenu.append(stop_oneconf_share_menuitem)
         self.oneconfcontrol.pack_start(oneconftoolbar, False, False, 1)
@@ -166,7 +185,8 @@ class InstalledPane(SoftwarePane, CategoriesParser):
         self.notebook.append_page(self.box_app_list, Gtk.Label(label="list"))
 
         # details
-        self.notebook.append_page(self.scroll_details, Gtk.Label(label="details"))
+        self.notebook.append_page(self.scroll_details,
+            Gtk.Label(label="details"))
         # initial refresh
         self.state.search_term = ""
 
@@ -177,28 +197,26 @@ class InstalledPane(SoftwarePane, CategoriesParser):
         self.treefilter.set_visible_func(self._row_visibility_func,
                                          AppTreeStore.COL_ROW_DATA)
         self.app_view.set_model(self.treefilter)
-        self.app_view.tree_view.connect("row-collapsed", self._on_row_collapsed)
+        self.app_view.tree_view.connect("row-collapsed",
+            self._on_row_collapsed)
 
-        self._all_cats = self.parse_applications_menu('/usr/share/app-install')
+        self._all_cats = self.parse_applications_menu(APP_INSTALL_PATH)
         self._all_cats = categories_sorted_by_name(self._all_cats)
-        
+
         # we do not support the search aid feature in the installedview
         self.box_app_list.remove(self.search_aid)
-        
-        # create a local spinner notebook for the installed view
-        self.installed_spinner_view = SpinnerView()
-        self.installed_spinner_notebook = Gtk.Notebook()
-        self.installed_spinner_notebook.set_show_tabs(False)
-        self.installed_spinner_notebook.set_show_border(False)
-        self.installed_spinner_notebook.append_page(self.installed_spinner_view, None)
+
+        # remove here
         self.box_app_list.remove(self.app_view)
-        self.installed_spinner_notebook.append_page(self.app_view, None)
-        
+
+        # create a local spinner notebook for the installed view
+        self.installed_spinner_notebook = SpinnerNotebook(self.app_view)
+
         self.computerpane.pack2(self.installed_spinner_notebook, True, True)
         self.show_installed_view_spinner()
-        
+
         self.show_all()
-        
+
         # initialize view to hide the oneconf computer selector
         self.oneconf_viewpickler.select_first()
         self.oneconfcontrol.hide()
@@ -206,28 +224,26 @@ class InstalledPane(SoftwarePane, CategoriesParser):
         # hacky, hide the header
         self.app_view.header_hbox.hide()
 
+        self.hide_appview_spinner()
+
+        # keep track of the current view by tracking its origin
+        self.current_displayed_origin = None
+
         # now we are initialized
         self.emit("installed-pane-created")
-        
+
         self.view_initialized = True
         return False
-        
+
     def show_installed_view_spinner(self):
         """ display the local spinner for the installed view panel """
-        self.installed_spinner_view.stop()
-        self.installed_spinner_notebook.set_current_page(self.PAGE_SPINNER)
-        # "mask" the spinner view momentarily to prevent it from flashing into
-        # view in the case of short delays where it isn't actually needed
-        GObject.timeout_add(100, self._unmask_installed_view_spinner)
-        
-    def _unmask_installed_view_spinner(self):
-        self.installed_spinner_view.start()
-        return False
-        
+        if self.installed_spinner_notebook:
+            self.installed_spinner_notebook.show_spinner()
+
     def hide_installed_view_spinner(self):
         """ hide the local spinner for the installed view panel """
-        self.installed_spinner_notebook.set_current_page(self.PAGE_INSTALLED)
-        self.installed_spinner_view.stop()
+        if self.installed_spinner_notebook:
+            self.installed_spinner_notebook.hide_spinner()
 
     def _selected_computer_changed(self, oneconf_pickler, hostid, hostname):
         if self.current_hostid == hostid:
@@ -235,13 +251,19 @@ class InstalledPane(SoftwarePane, CategoriesParser):
         LOG.debug("Selected computer changed to %s (%s)" % (hostid, hostname))
         self.current_hostid = hostid
         self.current_hostname = hostname
+        menuitem = self.oneconfproperty.get_menu().get_children()[0]
         if self.current_hostid:
-            (self.oneconf_additional_pkg, self.oneconf_missing_pkg) = self.oneconf_handler.oneconf.diff(self.current_hostid, '')
+            diff = self.oneconf_handler.oneconf.diff(self.current_hostid, '')
+            self.oneconf_additional_pkg, self.oneconf_missing_pkg = diff
+            stopsync_hostname = self.current_hostname
             # FIXME for P: oneconf views don't support search
-            if self.state.search_term:  
+            if self.state.search_term:
                 self._search()
         else:
+            stopsync_hostname = platform.node()
             self.searchentry.show()
+        menuitem.set_label(self.stopsync_label %
+            stopsync_hostname.encode('utf-8'))
         self.refresh_apps()
 
     def _last_time_sync_oneconf_changed(self, oneconf_handler, msg):
@@ -256,17 +278,22 @@ class InstalledPane(SoftwarePane, CategoriesParser):
             self.oneconf_viewpickler.select_first()
             self.oneconfcontrol.hide()
 
-    def _on_stop_showing_oneconf_clicked(self, widget):
-        LOG.debug("Stop sharing the current computer inventory")
-        self.oneconf_handler.sync_between_computers(False)
-        
+    def _on_stop_oneconf_hostshare_clicked(self, widget):
+        LOG.debug("Stop sharing inventory for %s" % self.current_hostname)
+        self.oneconf_handler.sync_between_computers(False, self.current_hostid)
+        # stop sharing another host than the local one.
+        if self.current_hostid:
+            self.oneconf_viewpickler.remove_computer(self.current_hostid)
+            self.oneconf_viewpickler.select_first()
+
     def _current_inventory_need_refresh(self, oneconfviews):
         if self.current_hostid:
-            (self.oneconf_additional_pkg, self.oneconf_missing_pkg) = self.oneconf_handler.oneconf.diff(self.current_hostid, '')
+            diff = self.oneconf_handler.oneconf.diff(self.current_hostid, '')
+            self.oneconf_additional_pkg, self.oneconf_missing_pkg = diff
         self.refresh_apps()
 
     def _on_row_collapsed(self, view, it, path):
-        return
+        pass
 
     def _row_visibility_func(self, model, it, col):
         row = model.get_value(it, col)
@@ -278,16 +305,18 @@ class InstalledPane(SoftwarePane, CategoriesParser):
         elif isinstance(row, CategoryRowReference):
             return row.untranslated_name in self.visible_cats.keys()
 
-        elif row is None: return False
+        elif row is None:
+            return False
 
         return row.get_docid() in self.visible_docids
 
     def _use_category(self, cat):
         # System cat is large and slow to search, filter it in default mode
 
-        if ('carousel-only' in cat.flags or 
+        if ('carousel-only' in cat.flags or
             ((self.nonapps_visible == NonAppVisibility.NEVER_VISIBLE)
-            and cat.untranslated_name == 'System')): return False
+            and cat.untranslated_name == 'System')):
+                return False
 
         return True
 
@@ -295,44 +324,82 @@ class InstalledPane(SoftwarePane, CategoriesParser):
     def _hide_nonapp_pkgs(self):
         self.nonapps_visible = NonAppVisibility.NEVER_VISIBLE
         self.refresh_apps()
+        return True
+
+    def _save_treeview_state(self):
+        # store the state
+        expanded_rows = []
+        self.app_view.tree_view.map_expanded_rows(
+            lambda view, path, data: expanded_rows.append(path.to_string()),
+            None)
+        va = self.app_view.tree_view_scroll.get_vadjustment()
+        if va:
+            vadj = va.get_value()
+        else:
+            vadj = 0
+        return expanded_rows, vadj
+
+    def _restore_treeview_state(self, state):
+        expanded_rows, vadj = state
+        for ind in expanded_rows:
+            path = Gtk.TreePath.new_from_string(ind)
+            self.app_view.tree_view.expand_row(path, False)
+        va = self.app_view.tree_view_scroll.get_vadjustment()
+        if va:
+            va.set_lower(vadj)
+            va.set_value(vadj)
 
     #~ @interrupt_build_and_wait
-    def _build_categorised_installedview(self):
+    def _build_categorised_installedview(self, keep_state=False):
         LOG.debug('Rebuilding categorised installedview...')
-        
+
         # display the busy cursor and a local spinner while we build the view
         window = self.get_window()
         if window:
             window.set_cursor(self.busy_cursor)
         self.show_installed_view_spinner()
-        
-        model = self.base_model # base model not treefilter
+
+        if keep_state:
+            treeview_state = self._save_treeview_state()
+
+        # disconnect the model to avoid e.g. updates of "cursor-changed"
+        #  AppTreeView.expand_path while the model is in rebuild-flux
+        self.app_view.set_model(None)
+
+        model = self.base_model  # base model not treefilter
         model.clear()
 
+        def profiled_rebuild_categorised_view():
+            with ExecutionTime("rebuild_categorized_view"):
+                rebuild_categorised_view()
+
         def rebuild_categorised_view():
             self.cat_docid_map = {}
             enq = self.enquirer
-            
+
             i = 0
-            
+
             while Gtk.events_pending():
                 Gtk.main_iteration()
 
             xfilter = AppFilter(self.db, self.cache)
             xfilter.set_installed_only(True)
+
             for cat in self._all_cats:
                 # for each category do category query and append as a new
                 # node to tree_view
-                if not self._use_category(cat): continue
+                if not self._use_category(cat):
+                    continue
                 query = self.get_query_for_cat(cat)
-                LOG.debug("xfilter.installed_only: %s" % xfilter.installed_only)
+                LOG.debug("xfilter.installed_only: %s" %
+                    xfilter.installed_only)
                 enq.set_query(query,
                               sortmode=SortMethods.BY_ALPHABET,
                               nonapps_visible=self.nonapps_visible,
                               filter=xfilter,
                               nonblocking_load=False,
-                              persistent_duplicate_filter=(i>0))
-                              
+                              persistent_duplicate_filter=(i > 0))
+
                 L = len(enq.matches)
                 if L:
                     i += L
@@ -340,13 +407,13 @@ class InstalledPane(SoftwarePane, CategoriesParser):
                     self.cat_docid_map[cat.untranslated_name] = \
                                         set([doc.get_docid() for doc in docs])
                     model.set_category_documents(cat, docs)
-                    
+
             while Gtk.events_pending():
                 Gtk.main_iteration()
 
             # check for uncategorised pkgs
             if self.state.channel:
-                self._run_channel_enquirer(persistent_duplicate_filter=(i>0))
+                self._run_channel_enquirer(persistent_duplicate_filter=(i > 0))
                 L = len(enq.matches)
                 if L:
                     # some foo for channels
@@ -357,7 +424,8 @@ class InstalledPane(SoftwarePane, CategoriesParser):
                         channel_name = self.state.channel.display_name
                     docs = enq.get_documents()
                     tag = channel_name or 'Uncategorized'
-                    self.cat_docid_map[tag] = set([doc.get_docid() for doc in docs])
+                    self.cat_docid_map[tag] = set(
+                        [doc.get_docid() for doc in docs])
                     model.set_nocategory_documents(docs, untranslated_name=tag,
                                                    display_name=channel_name)
                     i += L
@@ -370,17 +438,19 @@ class InstalledPane(SoftwarePane, CategoriesParser):
 
             # cache the installed app count
             self.installed_count = i
-            self.app_view._append_appcount(self.installed_count, mode=AppView.INSTALLED_MODE)
-            
+            self.app_view._append_appcount(self.installed_count,
+                mode=AppView.INSTALLED_MODE)
+
+            self.app_view.set_model(self.treefilter)
+            if keep_state:
+                self._restore_treeview_state(treeview_state)
+
             # hide the local spinner
             self.hide_installed_view_spinner()
-            
-            # hide the main spinner (if it's showing)
-            self.hide_appview_spinner()
-            
+
             if window:
                 window.set_cursor(None)
-            
+
             # reapply search if needed
             if self.state.search_term:
                 self._do_search(self.state.search_term)
@@ -388,26 +458,36 @@ class InstalledPane(SoftwarePane, CategoriesParser):
             self.emit("app-list-changed", i)
             return
 
-        GObject.idle_add(rebuild_categorised_view)
-        return
-        
-    def _build_oneconfview(self):
+        GObject.idle_add(profiled_rebuild_categorised_view)
+
+    def _build_oneconfview(self, keep_state=False):
         LOG.debug('Rebuilding oneconfview for %s...' % self.current_hostid)
-        
+
         # display the busy cursor and the local spinner while we build the view
         window = self.get_window()
         if window:
             window.set_cursor(self.busy_cursor)
         self.show_installed_view_spinner()
-        
-        model = self.base_model # base model not treefilter
+
+        if keep_state:
+            treeview_state = self._save_treeview_state()
+
+        # disconnect the model to avoid e.g. updates of "cursor-changed"
+        #  AppTreeView.expand_path while the model is in rebuild-flux
+        self.app_view.set_model(None)
+
+        model = self.base_model  # base model not treefilter
         model.clear()
 
+        def profiled_rebuild_oneconfview():
+            with ExecutionTime("rebuild_oneconfview"):
+                rebuild_oneconfview()
+
         def rebuild_oneconfview():
-        
+
             # FIXME for P: hide the search entry
             self.searchentry.hide()
-            
+
             self.cat_docid_map = {}
             enq = self.enquirer
             query = xapian.Query("")
@@ -417,30 +497,33 @@ class InstalledPane(SoftwarePane, CategoriesParser):
                                      self.state.channel.query)
 
             i = 0
-            
+
             # First search: missing apps only
             xfilter = AppFilter(self.db, self.cache)
             xfilter.set_restricted_list(self.oneconf_additional_pkg)
             xfilter.set_not_installed_only(True)
-            
+
             enq.set_query(query,
                           sortmode=SortMethods.BY_ALPHABET,
                           nonapps_visible=self.nonapps_visible,
                           filter=xfilter,
-                          nonblocking_load=True, # we don't block this one for better oneconf responsiveness
-                          persistent_duplicate_filter=(i>0))
+                          nonblocking_load=True,  # we don't block this one for
+                                                # better oneconf responsiveness
+                          persistent_duplicate_filter=(i > 0))
 
             L = len(enq.matches)
 
             if L:
-                cat_title = ngettext(u'%(amount)s item on “%(machine)s” not on this computer',
-                                     u'%(amount)s items on “%(machine)s” not on this computer',
-                                     L) % { 'amount' : L, 'machine': self.current_hostname}
+                cat_title = utf8(ngettext(
+                    u'%(amount)s item on “%(machine)s” not on this computer',
+                    u'%(amount)s items on “%(machine)s” not on this computer',
+                    L)) % {'amount': L, 'machine': utf8(self.current_hostname)}
                 i += L
                 docs = enq.get_documents()
-                self.cat_docid_map["missingpkg"] = set([doc.get_docid() for doc in docs])
-                model.set_nocategory_documents(docs, untranslated_name="additionalpkg",
-                                               display_name=cat_title)
+                self.cat_docid_map["missingpkg"] = set(
+                    [doc.get_docid() for doc in docs])
+                model.set_nocategory_documents(docs,
+                    untranslated_name="additionalpkg", display_name=cat_title)
 
             # Second search: additional apps
             xfilter.set_restricted_list(self.oneconf_missing_pkg)
@@ -451,18 +534,20 @@ class InstalledPane(SoftwarePane, CategoriesParser):
                           nonapps_visible=self.nonapps_visible,
                           filter=xfilter,
                           nonblocking_load=False,
-                          persistent_duplicate_filter=(i>0))
+                          persistent_duplicate_filter=(i > 0))
 
             L = len(enq.matches)
             if L:
-                cat_title = ngettext(u'%(amount)s item on this computer not on “%(machine)s”',
-                                     u'%(amount)s items on this computer not on “%(machine)s”',
-                                     L) % { 'amount' : L, 'machine': self.current_hostname}
+                cat_title = utf8(ngettext(
+                    u'%(amount)s item on this computer not on “%(machine)s”',
+                    '%(amount)s items on this computer not on “%(machine)s”',
+                    L)) % {'amount': L, 'machine': utf8(self.current_hostname)}
                 i += L
                 docs = enq.get_documents()
-                self.cat_docid_map["additionalpkg"] = set([doc.get_docid() for doc in docs])
-                model.set_nocategory_documents(docs, untranslated_name="additionalpkg",
-                                               display_name=cat_title)
+                self.cat_docid_map["additionalpkg"] = set(
+                    [doc.get_docid() for doc in docs])
+                model.set_nocategory_documents(docs,
+                    untranslated_name="additionalpkg", display_name=cat_title)
 
             if i:
                 self.app_view.tree_view.set_cursor(Gtk.TreePath(),
@@ -472,31 +557,34 @@ class InstalledPane(SoftwarePane, CategoriesParser):
 
             # cache the installed app count
             self.installed_count = i
-            self.app_view._append_appcount(self.installed_count, mode=AppView.DIFF_MODE)
-                
+            self.app_view._append_appcount(self.installed_count,
+                mode=AppView.DIFF_MODE)
+
+            self.app_view.set_model(self.treefilter)
+            if keep_state:
+                self._restore_treeview_state(treeview_state)
+
             # hide the local spinner
             self.hide_installed_view_spinner()
-            
+
             if window:
                 window.set_cursor(None)
-            
+
             self.emit("app-list-changed", i)
             return
 
-        GObject.idle_add(rebuild_oneconfview)
-        return
+        GObject.idle_add(profiled_rebuild_oneconfview)
 
     def _check_expand(self):
         it = self.treefilter.get_iter_first()
         while it:
             path = self.treefilter.get_path(it)
-            if self.state.search_term:# or path in self._user_expanded_paths:
+            if self.state.search_term:  # or path in self._user_expanded_paths:
                 self.app_view.tree_view.expand_row(path, False)
             else:
                 self.app_view.tree_view.collapse_row(path)
 
             it = self.treefilter.iter_next(it)
-        return
 
     def _do_search(self, terms):
         self.state.search_term = terms
@@ -506,7 +594,7 @@ class InstalledPane(SoftwarePane, CategoriesParser):
                                 nonapps_visible=self.nonapps_visible,
                                 filter=xfilter,
                                 nonblocking_load=True)
-        
+
         self.visible_docids = self.enquirer.get_docids()
         self.visible_cats = self._get_vis_cats(self.visible_docids)
         self.treefilter.refilter()
@@ -538,7 +626,6 @@ class InstalledPane(SoftwarePane, CategoriesParser):
             self.emit("app-list-changed", 0)
         elif self.state.search_term != terms:
             self._do_search(terms)
-        return
 
     def get_query(self):
         # search terms
@@ -558,11 +645,11 @@ class InstalledPane(SoftwarePane, CategoriesParser):
     def refresh_apps(self, *args, **kwargs):
         """refresh the applist and update the navigation bar """
         logging.debug("installedpane refresh_apps")
+        keep_state = kwargs.get("keep_state", False)
         if self.current_hostid:
-            self._build_oneconfview()
+            self._build_oneconfview(keep_state)
         else:
-            self._build_categorised_installedview()
-        return
+            self._build_categorised_installedview(keep_state)
 
     def _clear_search(self):
         # remove the details and clear the search
@@ -576,7 +663,6 @@ class InstalledPane(SoftwarePane, CategoriesParser):
         self.state.search_term = terms
         self.notebook.set_current_page(InstalledPane.Pages.LIST)
         self.hide_installed_view_spinner()
-        return
 
     def _get_vis_cats(self, visids):
         vis_cats = {}
@@ -590,10 +676,18 @@ class InstalledPane(SoftwarePane, CategoriesParser):
         self.app_view._append_appcount(appcount, mode=AppView.DIFF_MODE)
         return vis_cats
 
-    def on_db_reopen(self, db):
-        self.refresh_apps(rebuild=True)
+    def _refresh_on_cache_or_db_change(self):
+        self.refresh_apps(keep_state=True)
         self.app_details_view.refresh_app()
 
+    def on_db_reopen(self, db):
+        super(InstalledPane, self).on_db_reopen(db)
+        self._refresh_on_cache_or_db_change()
+
+    def on_cache_ready(self, cache):
+        LOG.debug("on_cache_ready")
+        self._refresh_on_cache_or_db_change()
+
     def on_application_selected(self, appview, app):
         """callback when an app is selected"""
         logging.debug("on_application_selected: '%s'" % app)
@@ -610,36 +704,45 @@ class InstalledPane(SoftwarePane, CategoriesParser):
             self.emit("app-list-changed", len(model))
         self.searchentry.show()
 
+    @wait_for_apt_cache_ready
     def display_overview_page(self, page, view_state):
         LOG.debug("view_state: %s" % view_state)
         if self.current_hostid:
-            # FIXME for P: oneconf views don't support search    
+            # FIXME for P: oneconf views don't support search
             # this one ensure that even when switching between pane, we
             # don't have the search item
             if self.state.search_term:
                 self._search()
             self._build_oneconfview()
-        else:
+        elif (view_state and
+              view_state.channel and
+              view_state.channel.origin is not self.current_displayed_origin):
+            # we don't need to refresh the full installed view every time it
+            # is displayed, so we check to see if we are viewing the same
+            # channel and if so we don't refresh the view, note that the view
+            # *is* is refreshed whenever the contents change and this is
+            # sufficient (see LP: #828887)
             self._build_categorised_installedview()
+            self.current_displayed_origin = view_state.channel.origin
 
-        if self.state.search_term:
-            self._search(self.state.search_term)
+            if self.state.search_term:
+                self._search(self.state.search_term)
         return True
 
     def get_current_app(self):
         """return the current active application object applicable
            to the context"""
         return self.current_appview_selection
-        
+
     def is_category_view_showing(self):
         # there is no category view in the installed pane
         return False
-        
+
     def is_applist_view_showing(self):
         """Return True if we are in the applist view """
         return (self.notebook.get_current_page() ==
                 InstalledPane.Pages.LIST)
-        
+
     def is_app_details_view_showing(self):
         """Return True if we are in the app_details view """
         return self.notebook.get_current_page() == InstalledPane.Pages.DETAILS
@@ -654,7 +757,7 @@ def get_test_window():
                                           )
     # needed because available pane will try to get it
     vm = get_test_gtk3_viewmanager()
-    vm # make pyflakes happy
+    vm  # make pyflakes happy
     db = get_test_db()
     cache = get_test_pkg_info()
     datadir = get_test_datadir()
@@ -673,8 +776,11 @@ def get_test_window():
     w.init_view()
 
     from softwarecenter.backend.channel import AllInstalledChannel
+    from softwarecenter.ui.gtk3.panes.softwarepane import DisplayState
     w.state.channel = AllInstalledChannel()
-    w.display_overview_page(None, None)
+    view_state = DisplayState()
+    view_state.channel = AllInstalledChannel()
+    w.display_overview_page(None, view_state)
 
     win.show()
     return win
@@ -683,4 +789,3 @@ def get_test_window():
 if __name__ == "__main__":
     win = get_test_window()
     Gtk.main()
-
