@@ -1,35 +1,36 @@
 #!/usr/bin/python
 
-import sys
-sys.path.insert(0,"../")
+from gi.repository import GObject
 
 import apt
-import apt_pkg
 import datetime
-import glib
 import logging
 import os
 import subprocess
+import sys
 import time
 import unittest
 
-from softwarecenter.apt.apthistory import AptHistory
+from testutils import setup_test_env
+setup_test_env()
+
+from softwarecenter.db.history_impl.apthistory import AptHistory
 from softwarecenter.utils import ExecutionTime
 
 class TestAptHistory(unittest.TestCase):
 
-    def _get_apt_history(self):
-        history = AptHistory(use_cache=False)
-        main_loop = glib.main_context_default()
-        while main_loop.pending():
-           main_loop.iteration()
-        return history
-
     def setUp(self):
         rundir = os.path.abspath(os.path.dirname(sys.argv[0]))
         self.basedir = os.path.join(rundir, "./data/apt-history")
-        apt_pkg.Config.set("Dir::Log", self.basedir)
-        #apt_pkg.Config.set("Dir::Log::History", "./)
+        apt.apt_pkg.Config.set("Dir::Log", self.basedir)
+        #apt_pkg.Config.set("Dir::Log::History", "./")
+
+    def _get_apt_history(self):
+        history = AptHistory(use_cache=False)
+        main_loop = GObject.main_context_default()
+        while main_loop.pending():
+           main_loop.iteration()
+        return history
 
     def test_history(self):
         history = self._get_apt_history()
@@ -75,10 +76,10 @@ class TestAptHistory(unittest.TestCase):
         history = self._get_apt_history()
         self.assertEqual(len(history.transactions), 186)
         self._generate_big_history_file(new_history)
-        timer_id = glib.timeout_add(100, self._glib_timeout)
+        timer_id = GObject.timeout_add(100, self._glib_timeout)
         with ExecutionTime("rescan %s byte file" % os.path.getsize(new_history+".gz")):
-            history.rescan(use_cache=False)
-        glib.source_remove(timer_id)
+            history._rescan(use_cache=False)
+        GObject.source_remove(timer_id)
         # verify rescan
         self.assertTrue(len(history.transactions) > 186)
         # check the timeouts
@@ -91,11 +92,11 @@ class TestAptHistory(unittest.TestCase):
 
     def test_no_history_log(self):
         # set to dir with no existing history.log
-        apt_pkg.Config.set("Dir::Log", "/")
+        apt.apt_pkg.Config.set("Dir::Log", "/")
         # this should not raise
         history = self._get_apt_history()
         self.assertEqual(history.transactions, [])
-        apt_pkg.Config.set("Dir::Log", self.basedir)
+        apt.apt_pkg.Config.set("Dir::Log", self.basedir)
 
 if __name__ == "__main__":
     logging.basicConfig(level=logging.DEBUG)
