@@ -1,11 +1,10 @@
 #!/usr/bin/python
 
-import sys
-import unittest
-sys.path.insert(0,"../")
-
 import apt
-from softwarecenter.apt.aptcache import AptCache
+import unittest
+
+from testutils import setup_test_env
+setup_test_env()
 
 class TestOrigins(unittest.TestCase):
     """ tests the origin code """
@@ -17,6 +16,7 @@ class TestOrigins(unittest.TestCase):
         cache.open()
         # PPA origin
         origins = cache["firefox-trunk"].candidate.origins
+        print origins
         self.assertEqual(origins[0].site, "ppa.launchpad.net")
         self.assertEqual(origins[0].origin, "LP-PPA-ubuntu-mozilla-daily")
         # archive origin
