@@ -4,17 +4,27 @@ import datetime
 import glob
 import multiprocessing
 import os
+import stat
 import subprocess
+import shutil
 import tempfile
 import time
 import unittest
 
+from mock import patch
+
 from testutils import setup_test_env
 setup_test_env()
+
 from softwarecenter.utils import (decode_xml_char_reference,
                                   release_filename_in_lists_from_deb_line,
                                   get_http_proxy_string_from_libproxy,
+                                  get_file_path_from_iconname,
+                                  get_oem_channel_descriptor,
                                   )
+from softwarecenter.testutils import (do_events,
+                                      get_test_gtk3_icon_cache,
+                                      )
 
 
 class TestSCUtils(unittest.TestCase):
@@ -129,6 +139,11 @@ class TestSCUtils(unittest.TestCase):
         uuid = get_uuid()
         self.assertTrue(uuid and len(uuid) > 0)
 
+    def test_clear_credentials(self):
+        from softwarecenter.utils import clear_token_from_ubuntu_sso_sync
+        clear_token_from_ubuntu_sso_sync("fo")
+        do_events()
+
     def test_make_string_from_list(self):
         from softwarecenter.utils import make_string_from_list
         base = "There was a problem posting this review to %s (omg!)"
@@ -152,7 +167,68 @@ class TestSCUtils(unittest.TestCase):
         self.assertEqual(
             make_string_from_list(base, l),
             "There was a problem posting this review to twister, factbook, identi.catz and baz (omg!)")
-        
+
+    def test_capitalize_first_word(self):
+        from softwarecenter.utils import capitalize_first_word
+        test_synopsis = "feature-rich console based todo list manager"
+        capitalized = capitalize_first_word(test_synopsis)
+        self.assertTrue(
+            capitalized == "Feature-rich console based todo list manager")
+        test_synopsis = "MPlayer's Movie Encoder"
+        capitalized = capitalize_first_word(test_synopsis)
+        self.assertTrue(
+            capitalized == "MPlayer's Movie Encoder")
+        # ensure it does not crash for empty strings, LP: #1002271
+        self.assertEqual(capitalize_first_word(""), "")
+            
+    def test_ensure_file_writable_and_delete_if_not(self):
+        from softwarecenter.utils import ensure_file_writable_and_delete_if_not
+        from tempfile import NamedTemporaryFile
+        # first test that a non-writable file (0400) is deleted
+        test_file_not_writeable = NamedTemporaryFile()
+        os.chmod(test_file_not_writeable.name, stat.S_IRUSR)
+        self.assertFalse(os.access(test_file_not_writeable.name, os.W_OK))
+        ensure_file_writable_and_delete_if_not(test_file_not_writeable.name)
+        self.assertFalse(os.path.exists(test_file_not_writeable.name))
+        # then test that a writable file (0600) is not deleted
+        test_file_writeable = NamedTemporaryFile()
+        os.chmod(test_file_writeable.name, stat.S_IRUSR|stat.S_IWUSR)
+        self.assertTrue(os.access(test_file_writeable.name, os.W_OK))
+        ensure_file_writable_and_delete_if_not(test_file_writeable.name)
+        self.assertTrue(os.path.exists(test_file_writeable.name))
+
+    def test_safe_makedirs(self):
+        from softwarecenter.utils import safe_makedirs
+        from tempfile import mkdtemp
+        tmp = mkdtemp()
+        # create base dir
+        target = os.path.join(tmp, "foo", "bar")
+        safe_makedirs(target)
+        # we need the patch to ensure that the code is actually executed
+        with patch("os.path.exists") as mock_:
+            mock_.return_value = False
+            self.assertTrue(os.path.isdir(target))
+            # ensure that creating the base dir again does not crash
+            safe_makedirs(target)
+            self.assertTrue(os.path.isdir(target))
+            # ensure we still get regular errors like permission denied
+            # (stat.S_IRUSR)
+            os.chmod(os.path.join(tmp, "foo"), 0400)
+            self.assertRaises(OSError, safe_makedirs, target)
+            # set back to stat.(S_IRUSR|S_IWUSR|S_IXUSR) to make rmtree work
+            os.chmod(os.path.join(tmp, "foo"), 0700)
+        # cleanup
+        shutil.rmtree(tmp)
+
+    def test_get_file_path_from_iconname(self):
+        icons = get_test_gtk3_icon_cache()
+        icon_path = get_file_path_from_iconname(
+                icons,
+                "softwarecenter")
+        self.assertEqual(
+                icon_path,
+                "/usr/share/icons/hicolor/48x48/apps/softwarecenter.svg")
+
 
 class TestExpungeCache(unittest.TestCase):
 
@@ -210,6 +286,10 @@ class TestExpungeCache(unittest.TestCase):
         # ensure that the second one was not called
         self.assertEqual(len(glob.glob(os.path.join(tmpdir, "marker.*"))), 1)
 
+    def test_oem_channel_extractor(self):
+        s = get_oem_channel_descriptor("./data/ubuntu_dist_channel")
+        self.assertEqual(s, "canonical-oem-watauga-precise-amd64-20120517-2")
+
 
 
 if __name__ == "__main__":
